# -*- coding: utf-8 -*-
# Copyright (C) 2006-2007  Vodafone España, S.A.
# Author:  Pablo Martí
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
OS Abstraction Layer

OS provides an abstraction layer so path differences between OSes/distros
won't affect VMCCdfL
"""
__version__ = "$Rev: 658 $"

import os
import stat

from twisted.python.procutils import which

from vmc.common.exceptions import OSNotRegisteredInOALError

def get_os_object():
    """
    Returns a C{BaseDistribution} object corresponding to current OS used
    
    It will return None if the OS/Distro is no registered of course
    """
    for klass in OSNAME_COLLECTION:
        instance = klass() 
        if instance.perform_check():
            return instance
    return None

def mark_for_oal(path):
    """
    mark_for_oal must be used as gettext's '_'.
    
    Whatever you want to abstract because is different between OSes/Distros
    must be marked with this command
    """
    distro = get_os_object()
    if not distro:
        raise OSNotRegisteredInOALError
    
    return distro.abstraction[path]

BASE_PATH_DICT = {
      'WVDIAL_AUTH_CONF': '/etc/ppp/peers/wvdial',
      'WVDIAL_CONN_SWITCH' : '-C',
      'WVDIAL_DEFAULT_PROFILE' : 'wvdial.default',
      'WVDIAL_PAP_PROFILE' : 'wvdial.only-pap',
      'WVDIAL_CHAP_PROFILE' : 'wvdial.only-chap',
      'gksudo_name' : 'gksudo',
      'PAP_SECRETS' : '/etc/ppp/pap-secrets',
      'CHAP_SECRETS' : '/etc/ppp/chap-secrets',
      'CFG_TEMPLATE' : 'vmc.cfg.tpl',
}

class BaseDistribution(object):
    """
    I am the base class all new OSes/Distros should inherit from
    
    I've got some fields that should be used to customise me:
     - __required__ : Name of the file that identifies this Distro/OS
     - abstraction : Don't touch this
     - customization : Dict with all the abstractions/transformations
     - requires_privileges : Set to True if its necessary to start the app
       with sudo
     - manage_secrets: Set to True if its necessary to manage
       /etc/ppp/{ch,p}ap-secrets
    """
    __required__ = 'no set'
    customization = None
    abstraction = BASE_PATH_DICT
    manage_secrets = True
    
    def __init__(self):
        self.pppd_is_suid = None
        self.check_if_pppd_is_suid()
        self.customize_myself()
    
    def check_if_pppd_is_suid(self):
        """
        Sets C{self.pppd_is_suid} to True if pppd is suid
        
        This should take into account SELinux stuff
        """
        try:
            pppd_path = which('pppd')[0]
        except IndexError:
            pppd_path = '/usr/sbin/pppd'
        
        smode = os.stat(pppd_path)[stat.ST_MODE]
        mode = str(stat.S_IMODE(smode))
        self.pppd_is_suid = len(mode) == 4
    
    def perform_check(self):
        """Returns C{True} if self.__required__ exists in the system"""
        return os.path.exists(self.__required__)
    
    def customize_myself(self):
        """Update C{self.abstraction} if necessary"""
        if self.customization:
            self.abstraction.update(self.customization)
        
        if os.environ.has_key('KDE_FULL_SESSION'):
            self.abstraction['gksudo_name'] == 'kdesu'
    
    def get_connection_args(self, wvdial_conf_path):
        """Returns the connection args customized for each OS/Distro"""
        wvdial_path = which('wvdial')[0]
        if self.pppd_is_suid:
            return [wvdial_path, mark_for_oal('WVDIAL_CONN_SWITCH'),
                wvdial_conf_path, 'connect']
        
        gksudo_name = self.abstraction['gksudo_name']
        gksudo_path = which(gksudo_name)[0]
        if gksudo_name == 'gksudo':
            return [gksudo_path, wvdial_path,
                    mark_for_oal('WVDIAL_CONN_SWITCH'),
                    wvdial_conf_path, 'connect']
        
        elif gksudo_name == 'gnomesu':
            args = " ".join([wvdial_path,
                                mark_for_oal('WVDIAL_CONN_SWITCH'),
                                wvdial_conf_path, 'connect'])
            return [gksudo_path, '-c', args]
        
        elif gksudo_name == 'gksu':
            args = " ".join([wvdial_path,
                                mark_for_oal('WVDIAL_CONN_SWITCH'),
                                wvdial_conf_path, 'connect'])
            return [gksudo_path, '-g', args]
        
        elif gksudo_name == 'kdesu':
            args = " ".join([wvdial_path,
                                mark_for_oal('WVDIAL_CONN_SWITCH'),
                                wvdial_conf_path, 'connect'])
            return [gksudo_path, '-c', args]
    
    def get_disconnection_args(self):
        killall_path = which('killall')[0]
        if self.pppd_is_suid:
            return [killall_path, 'pppd', 'wvdial']
        else:
            gksudo_name = self.abstraction['gksudo_name']
            gksudo_path = which(gksudo_name)[0]
            if gksudo_name == 'gksudo':
                return [gksudo_path, killall_path, 'pppd', 'wvdial']
            elif gksudo_name == 'gnomesu':
                args = " ".join([killall_path, 'pppd', 'wvdial'])
                return [gksudo_path, '-c', args]
            elif gksudo_name == 'gksu':
                args = " ".join([killall_path, 'pppd', 'wvdial'])
                return [gksudo_path, '-g', args]
            elif gksudo_name == 'kdesu':
                args = " ".join([killall_path, 'pppd', 'wvdial'])
                return [gksudo_path, '-c', args]
            
class DebianBasedDistribution(BaseDistribution):
    __required__ = '/etc/debian_version'

class FedoraBasedDistribution(BaseDistribution):
    __required__ = '/etc/fedora-release'
    customization = {
         'WVDIAL_CONN_SWITCH' : '--config',
         'WVDIAL_DEFAULT_PROFILE' : 'wvdial.default.fedora',
         'WVDIAL_PAP_PROFILE' : 'wvdial.only-pap.fedora',
         'WVDIAL_CHAP_PROFILE' : 'wvdial.only-chap.fedora',
         'gksudo_name' : 'gksu',
    }

class SuseBasedDistribution(BaseDistribution):
    __required__ = '/etc/SuSE-release'
    customization = {
         'WVDIAL_CONN_SWITCH' : '--config',
         'WVDIAL_DEFAULT_PROFILE' : 'wvdial.default.sled',
         'WVDIAL_PAP_PROFILE' : 'wvdial.only-pap.sled',
         'WVDIAL_CHAP_PROFILE' : 'wvdial.only-chap.sled',
         'gksudo_name' : 'gnomesu',
         'CFG_TEMPLATE' : 'vmc.cfg.tpl.sled',
    }
    manage_secrets = False

OSNAME_COLLECTION = [DebianBasedDistribution,
                     FedoraBasedDistribution,
                     SuseBasedDistribution]
