# -*- coding: utf-8 -*-
# Copyright (C) 2006-2007  Vodafone España, S.A.
# Author:  Pablo Martí
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
phonebook presents a uniform layer to deal with contacts from both SIM and DB
"""

__version__ = "$Rev: 600 $"

from twisted.internet import defer

import vmc.common.exceptions as ex
from vmc.common.persistent import Contact, ContactsManager, SIM_SIZE
from vmc.common.smsng.sms import pack_ucs2_bytes

def is_sim_contact(contact):
    """Returns True if C{contact} is a SIM contact"""
    return int(contact.index) < SIM_SIZE
        
class PhoneBook(object):
    """PhoneBook manages contacts from both SIM and DB"""
    
    def __init__(self, sconn):
        self.sconn = sconn
        self.cmanager = ContactsManager()
    
    def close(self):
        self.cmanager.close()
        del self.sconn
    
    def add_contact(self, contact, sim=False):
        deferred = defer.Deferred()
        def add_contact_cb(index):
            contact.index = int(index)
            deferred.callback(contact)
        
        def add_contact_eb(failure):
            failure.trap(ex.CMEErrorInvalidCharactersInDialString)
            deferred.errback(failure)
        
        if sim:
            self.sconn.add_contact(contact).addCallbacks(add_contact_cb,
                                                         add_contact_eb)
        else:
            self.cmanager.add_contact(contact).addCallback(add_contact_cb)
        
        return deferred
    
    def _find_contact_in_sim(self, pattern):
        deferred = defer.Deferred()
        def find_cb(response):
            result = []
            if response:
                for match in response:
                    index, number, cat, name = match.groups()
                    result.append(Contact(name, number, int(index)))
            
            deferred.callback(result)
        
        self.sconn.find_contact(pattern).addCallback(find_cb)
        return deferred
    
    def _find_contact_in_db(self, pattern):
        deferred = defer.Deferred()
        def find_cb(response):
            result = []
            if response:
                for match in response:
                    index, cat, name, number = match
                    result.append(Contact(name, number, int(index)))
            
            deferred.callback(result)
        
        self.cmanager.find_contacts(pattern).addCallback(find_cb)
        return deferred
    
    def find_contact(self, name=None, number=None):
        if not name and not number:
            return defer.fail()
        if name and number:
            return defer.fail()

        deferred = defer.Deferred()
        
        if name:
            def find_contact_cb(simcontacts):
                def final_cb(dbcontacts):
                    deferred.callback(simcontacts + dbcontacts)
                
                self._find_contact_in_db(name).addCallback(final_cb)
            
            self._find_contact_in_sim(
                    pack_ucs2_bytes(name)).addCallback(find_contact_cb)
        
        if number:
            # searching by name is pretty easy as the SIM allows to lookup
            # contacts by name. However searching by number is more difficult
            # as the SIM doesn't provides any facility for it. Thus we need
            # to get *all* contacts and iterate through them looking for 
            # a number that matches the pattern
            def get_contacts_cb(contacts):
                matches = [contact for contact in contacts
                               if contact.number == number]
                deferred.callback(matches)
            
            self.get_contacts().addCallback(get_contacts_cb)
        
        return deferred
    
    def get_contacts(self):
        deferred = defer.Deferred()
        def sim_cb(simcontacts):
            def final_cb(dbcontacts):
                response = dbcontacts + simcontacts
                deferred.callback(response)
            
            self.cmanager.read_contacts().addCallback(final_cb)
        self.sconn.get_all_contacts().addCallback(sim_cb)
        return deferred
    
    def delete_contacts(self, clist):
        deflist = []
        for contact in clist:
            deflist.append(self.delete_contact(contact))
            
        return defer.DeferredList(deflist)
    
    def delete_contact(self, contact):
        deferred = defer.Deferred()
        response_cb = lambda _: deferred.callback(None)
        
        if is_sim_contact(contact):
            self.sconn.delete_contact(contact.index).addCallback(response_cb)
        else:
            self.cmanager.delete_ids([contact.index]).addCallback(response_cb)
        
        return deferred
    
    def edit_contact(self, contact):
        deferred = defer.Deferred()
        response_cb = lambda _: deferred.callback(None)
        
        if is_sim_contact(contact):
            self.sconn.add_contact(contact).addCallback(response_cb)
        else:
            self.cmanager.update_contact(contact).addCallback(response_cb)
        
        return deferred
        