# -*- coding: utf-8 -*-
# Authors:  Pablo Martí, Thomas Lotze and Vivesto Solutions
#
# This module is the result of a merge between code owned by Thomas Lotze
# and Vivesto Solutions plus some improvements of my own. 
#
# This file should be licensed under the MPL and the ZPL. While former is
# GPL-compatible, the MPL isn't. The MPL allows you to modify files and add
# portions of them to files of your own as long as you keep the copyright,
# the license, and you gotta explain what you did. Both licenses allow you to
# do whatever you want with the code as long as you maintain licenses. I had
# them separated before, but adding stuff was just impossible. After the
# merge, I've added some stuff of my own, I guess that the copyright is
# shared. This should be probably better clarified with a lawyer
# but for now, I will just leave this header. I hope it explains the situation
# and gives credit to all the parties.
"""
SMS related classes and utilities
"""
__version__ = "$Rev: 581 $"

import codecs
import time
import datetime

MAX_LENGTH_7BIT = 160
MAX_LENGTH_8BIT = 140
MAX_LENGTH_UCS2 = 70

ucs2_encoder = codecs.getencoder("utf_16be")
ucs2_decoder = codecs.getdecoder("utf_16be")
hex_decoder = codecs.getdecoder("hex_codec")

def int2hexstr(i):
    """Returns the hex representation of C{i}"""
    return '%02X' % i
        
def swap(source):
    """
    Convert a string of numbers into the semi-octet representation
    described in GSM 03.40, 9.1.2.3"""
    out = ''
    if len(source) % 2:
        source += 'F'

    for i in range(0, len(source), 2):
        out += source[i + 1]
        out += source[i]

    return out

def pack_7bit_bytes(s):
    """pack_7bit_bytes("hellohello") => "E8329BFD4697D9EC37"

    Packs a series of 7-bit bytes into 8-bit bytes and then to
    ASCII strings as hexidecimal. See GSM 03.40, 9.1.2.1"""
    
    out = ''
    for i in range(0, len(s)):
        if i % 8 == 7:
            continue
        
        end = ord(s[i]) >> (i % 8)

        if i + 1 < len(s):        
            start = ord(s[i + 1]) << (7 - (i % 8))
            start = start & 0xFF
            out += int2hexstr(start | end)
        else:
            out += int2hexstr(end)

    return out

def unpack_7bit_bytes(octets):
    """Decode 7 bit character strings packed into 8 bit byte strings"""
    
    septets = []
    overflow = overflow_len = 0

    for value in octets:
        septets.append(value << overflow_len & 127 | overflow)
        overflow = value >> (7 - overflow_len)
        overflow_len += 1
        if overflow_len == 7:
            septets.append(overflow)
            overflow = overflow_len = 0

    return septets

def pack_8bit_bytes(s):
    return "".join([int2hexstr(ord(c)) for c in s])

def pack_ucs2_bytes(s):
    return "".join([int2hexstr(ord(c)) for c in ucs2_encoder(s)[0]])

def unpack_ucs2_bytes(s):
    octets = [ord(c) for c in hex_decoder(s)[0]]
    user_data = "".join(chr(o) for o in octets)
    return ucs2_decoder(user_data)[0]

UNKNOWN = 0
INTERNATIONAL = ISDNORTELEPHONE = 1
NATIONAL = 2

class GSMAddress(object):
    
    def __init__(self, number, numbering_plan_id=ISDNORTELEPHONE):
        if number.startswith('+'):
            self.type_of_number = INTERNATIONAL
            self.number = number[1:]
        else:
            self.type_of_number = UNKNOWN
            self.number = number
        
        self.numbering_plan_id = numbering_plan_id

    def serialize(self):
        result = []
        result.append(int2hexstr(len(self.number)))

        # GSM 03.40, section 9.1.2.5
        header = 0x80

        header = header | (self.type_of_number << 4)
        header = header | self.numbering_plan_id
        
        result.append(int2hexstr(header))
        result.append(swap(self.number))
        
        return "".join(result)

DEFAULT_ALPHABET = 0x00
EIGHT_BIT = 0x04
UCS2 = 0x08

def gsm_decoder(septets):
    """Decode septets to unicode by the GSM default alphabet.

    septets: iterable of int

    returns unicode
    """
    septets_iter = iter(septets)
    resp = []
    for s in septets_iter:
        if s != 27:
            resp.append(GSM_DEFAULT_TABLE[s])
        else:
            resp.append(GSM_EXTENSION_TABLE[septets_iter.next()])
    return u"".join(resp)

GSM_DEFAULT_TABLE = u"\
@£$¥èéùìòÇ\nØø\rÅå\
Δ_ΦΓΛΩΠΨΣΘΞ ÆæßÉ\
 !\"#¤%&'()*+,-./\
0123456789:;<=>?\
¡ABCDEFGHIJKLMNO\
PQRSTUVWXYZÄÖÑÜ§\
¿abcdefghijklmno\
pqrstuvwxyzäöñüà\
"

class FallbackDict(dict):

    def __init__(self, fallback, *args, **kwargs):
        super(FallbackDict, self).__init__(*args, **kwargs)
        self.__fallback = fallback
        
    def __missing__(self, key):
        return self.__fallback[key]

GSM_EXTENSION_TABLE = FallbackDict(GSM_DEFAULT_TABLE, {
    10: u"\f",
    20: u"^",
    27: u" ",
    40: u"{",
    41: u"}",
    47: u"\\",
    60: u"[",
    61: u"~",
    62: u"]",
    64: u"|",
    101: u"€",
    })

class ShortMessage(object):
    """I represent a SMS in the system"""
    
    def __init__(self, number=None, text=None,
                 datetime=None, where=None, index=None):
        self.number = number
        self.text = text
        self.datetime = datetime
        self.where = where
        self.index = index
    
    def __str__(self):
        return self.__class__.__name__ + \
             "<%s instance: index=%d, number = %s, text: '%s'>" % \
             (self.__class__.__name__, self.index, self.number, self.text)
    
    def __repr__(self):
        return self.__str__()
    
    def get_localised_date(self):
        return self.datetime and time.strftime("%c",
                           self.datetime.timetuple()) or None
    def get_number(self):
        return self.number
    
    def get_text(self):
        return self.text
    
    def user_data_from_octets(self, flags, octets, coding_scheme):
        user_data_len = octets.pop(0)

        if flags & 0x40: # user data header indicator
            header_len8 = octets[0] + 1
            self.header = octets[1:header_len8]
        else:
            header_len8 = 0

        compressed = (coding_scheme & 0xE0 == 0x20)
        if compressed:
            raise NotImplementedError

        if (coding_scheme & 0xEC == 0x00 or     # general data coding
            coding_scheme & 0xE0 == 0xC0 or     # message waiting indication
            coding_scheme & 0xF4 == 0xF0):      # data coding/message class
            coding = DEFAULT_ALPHABET
        elif (coding_scheme & 0xEC == 0x04 or   # general data coding
              coding_scheme & 0xF4 == 0xF4):    # data coding/message class
            coding = EIGHT_BIT
        elif (coding_scheme & 0xEC == 0x08 or   # general data coding
              coding_scheme & 0xF0 == 0xE0):    # message waiting indication
            coding = UCS2
        else:
            raise ValueError("Unsupported coding scheme: %d" % coding_scheme)

        if coding == DEFAULT_ALPHABET:
            header_len7 = (header_len8 * 8 + 6) / 7 # ceil(int*8/7)
            septets = unpack_7bit_bytes(octets)[header_len7:user_data_len]
            self.text = gsm_decoder(septets)
        else:
            octets = octets[header_len8:user_data_len]
            user_data = "".join(chr(o) for o in octets)
            if coding == UCS2:
                self.text = ucs2_decoder(user_data)[0]
            else:
                self.text = user_data

class ShortMessageDeliver(ShortMessage):

    def protocol_data_from_octets(self, flags, octets):
        self.address = gsm_address_from_octets(octets)
        del octets[0] # protocol identifier
        coding_scheme = octets.pop(0)
        self.datetime = datetime_from_octets(octets)

        return coding_scheme
    
    def get_number(self):
        if self.address.type_of_number == INTERNATIONAL \
            or self.address.type_of_number == UNKNOWN:
            return '+' + self.address.number
        else:
            return self.address.number

class ShortMessageSubmit(ShortMessage):
    
    def __init__(self, number, text, smsc=None, **kwds):
        ShortMessage.__init__(self, number, text, **kwds)
        self.address = GSMAddress(number)
        self.smsc = smsc
        self.encoding = None
        self.user_data = None
        self.__set_text(text)
    
    def __set_text(self, text):
        try:        
            sms_text = text.encode('sms-default')
            self.user_data_length = len(sms_text)
            self.user_data = pack_7bit_bytes(sms_text)
            self.encoding = DEFAULT_ALPHABET
        except UnicodeError, e:
            self.user_data = pack_ucs2_bytes(text)
            self.user_data_length = len(self.user_data) / 2
            self.encoding = UCS2
        
    def __len__(self):
        # FIXME: Return number of SMS
        return 1
    
    def protocol_data_from_octets(self, flags, octets):
        del octets[0] # TP message reference
        self.address = gsm_address_from_octets(octets)
        del octets[0] # protocol identifier
        coding_scheme = octets.pop(0)

        # TP validity period
        if flags & 0x08: # enhanced or absolute format
            del octets[:7]
        elif flags & 0x10: # relative format
            del octets[0]

        return coding_scheme
    
    def get_number(self):
        if self.address.type_of_number == INTERNATIONAL \
            or self.address.type_of_number == UNKNOWN:
            return '+' + self.address.number
        else:
            return self.address.number
    
    def get_text(self):
        return self.text
    
    def get_pdu_len(self):
        lenpdu = len(self.get_pdu())
        return (lenpdu/2) - self.get_smsc_len() - 1
    
    def get_smsc_len(self):
        swapped_smsc = swap(self.smsc[1:])
        swapped_smsc = '91' + swapped_smsc
        return len(swapped_smsc) / 2
        
    def get_pdu(self):
        """Returns the PDU of the message"""
        pdu = []
        assert self.smsc.startswith('+')
        swapped_smsc = swap(self.smsc[1:])
        swapped_smsc = '91' + swapped_smsc
        
        pdu.append(int2hexstr(self.get_smsc_len()))
        pdu.append(swapped_smsc)
        
        # TP-Message-Type-Indicator = 01
        # header = header | 0x01

        # TP-Reject-Duplicates = 0

        # TP-Validity-Period-Format = 00 (TP-VP field not present)

        # TP-Status-Report-Request = 0

        # TP-User-Data-Header-Indicator

        # TP-Reply-Path = 0
        #pdu.append(int2hexstr(header))
        pdu.append('11')
        # TP-Message-Reference = 0
        pdu.append(int2hexstr(0))
        
        # TP-Destination-Address
        pdu.append(self.address.serialize())

        # TP-Protocol-Identifier
        pdu.append(int2hexstr(0))

        # TP-Data-Coding-Scheme
        dcs = 0
        if self.encoding == DEFAULT_ALPHABET:
            dcs = dcs | 0x00
        else:
            dcs = dcs | 0x08
        
        pdu.append(int2hexstr(dcs))
        
        # TP-Validity-Period (skipped)
        pdu.append('AA')
        
        # TP-User-Data-Length
        pdu.append(int2hexstr(self.user_data_length))
        
        # TP-User-Data
        pdu.append(self.user_data)
        
        return "".join(pdu)

# RECV PART

def gsm_address_from_octets(octets):
    """Read a gsm address from octets.

    Consumes those octets holding the address field.

    returns GSMAddress
    """
    address_len = octets.pop(0)
    address_type = octets.pop(0)
    number = ""
    while len(number) < address_len:
        o = octets.pop(0)
        number += str(o & 15) + str(o >> 4)
    number = number[:address_len]
    return GSMAddress(number, address_type)

def datetime_from_octets(octets):
    """
    Returns a datetime object from octets

    Consumes those octets holding the timestamp.

    returns naive datetime.datetime instance relative to UTC
    """
    values = [(o & 15) * 10 + (o >> 4) for o in octets[:7]]
    del octets[:7]
    year = values[0]
    if year < 68:
        year = year + 2000 # YK2 Bug here
    result = datetime.datetime(year, *values[1:6])
    zone = values[-1]
    delta = datetime.timedelta(minutes=(zone & 127) * 15)
    if zone & 128:
        result -= delta
    else:
        result += delta
    
    return result

def pdu_to_message(pdu):
    octets = [ord(c) for c in hex_decoder(pdu)[0]]
    
    smsc_len = octets.pop(0)
    del octets[0] # delete smsc address type
    del octets[:smsc_len - 1] # delete smsc address
    
    flags = octets.pop(0)
    sms = (flags & 3) and ShortMessageSubmit() or ShortMessageDeliver()
    coding_scheme = sms.protocol_data_from_octets(flags, octets)
    sms.user_data_from_octets(flags, octets, coding_scheme)

    return sms

def message_to_pdu(sms):
    return sms.get_pdu()

def check_if_ucs2(text):
    try:
        text.encode('sms-default')
    except UnicodeError, e:
        return True
    else:
        return False
    