# -*- coding: utf-8 -*-
# Copyright (C) 2006-2007  Vodafone España, S.A.
# Author:  Pablo Martí
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
Stuff used at startup
"""
__version__ = "$Rev: 658 $"

import os
import re
import stat
import grp
import pwd
import errno
import shutil

from twisted.application.service import Service
from twisted.internet.serialport import SerialPort
from twisted.internet import reactor
from twisted.python import log

import vmc.utils.globals
import vmc.utils.dialogs
from vmc.common.protocol import SIMCardConnection
from vmc.common.middleware import SIMCardConnAdapter
import vmc.common.exceptions as ex
from vmc.common.config import VMCConfig
from vmc.common.hardware import DeviceListener
from vmc.common.shutdown import shutdown_core
from vmc.common.oal import mark_for_oal

from vmc.controllers.initialconf import DeviceConfController
from vmc.controllers.splash import SplashController
from vmc.controllers.pin import (AskPUKAndExecuteFuncController,
                             AskPINAndExecuteFuncController)
from vmc.gtkmvc import Model
from vmc.models.pin import AskPINModel
from vmc.models.initialconf import DeviceConfModel
from vmc.views.initialconf import DeviceConfView
from vmc.views.pin import AskPUKView, AskPINView
from vmc.views.splash import SplashView

from vmc.common.encoding import _

class StartupController(object):
    """
    I control the startup process, basically I maintain a counter with
    the number of common.exception.SIMFailureError exceptions received
    
    If that number is greater than 3 that means that there is no SIM
    inserted in the 3G device. I exist because there are some 3G devices
    that never inform you that there's no SIM inserted, they will just
    whine about a C{common.exceptions.SIMFailureError}
    """
    def __init__(self, device):
        self.device = device
        self.num_sim_failure_errors = 0
        self.sview = None
        self.dev_listener = None
        self.start_gui(device)
    
    def _init_sim(self):
        def reset_settings_cb(ignored): pass
        def reset_settings_eb(failure):
            log.err(failure)
        
        self.device.sconn.reset_settings().addCallbacks(reset_settings_cb,
                                                        reset_settings_eb)
        
        def disable_echo_cb(_): pass
        def disable_echo_eb(failure):
            failure.trap(ex.CMEErrorOperationNotAllowed)
            log.err(failure, _(
"""
If you own a Huawei E220 device you can safely ignore this error, otherwise
send info your device and this traceback to the devs"""))
        self.device.sconn.disable_echo().addCallbacks(disable_echo_cb,
                                                      disable_echo_eb)

    def start_gui(self, device):
        """I start the application"""
        # avoid gnome warnings
        import gnome
        import vmc.utils.globals
        gnome.init(vmc.utils.globals.APP_LONG_NAME, __version__)
        
        self.dev_listener = DeviceListener(self.device)
        
        sctrl = SplashController(Model())
        self.sview = SplashView(sctrl)
        
        self.sview.show()
        self.sview.set_fraction(.1)
        
        # check if this is the first time we run VMC
        try:
            vomcfg = VMCConfig()
        except IOError: # First time
            self.sview.set_text(_('Initial setup...'))
            self.sview.set_fraction(.1)
            
            import vmc.utils.utilities
            vmc.utils.utilities.initial_user_setup().addCallback(
                                                    self.second_step)
        else:
            self.second_step()
    
    def second_step(self, ignored=None):
        from vmc.common.wvdialprofile import (link_configuration,
                                              migrate_profiles)
        vomcfg = VMCConfig()
        # between 0.9.6 and 0.9.7 we did some changes in the wvdial profiles
        # and they need to be migrated, we don't want to hassle the user with
        # a tool because they probably won't use it. So we do this
        # automatically, the only way we have to check wether we are living
        # in a 0.9.6 system migrated to 0.9.7 is to check for the presence
        # of the 'exit_without_confirmation' key in the config.
        from ConfigParser import NoOptionError
        try:
            result = vomcfg.get('preferences', 'exit_without_confirmation')
        except NoOptionError:
            migrate_profiles()
            vomcfg.set('preferences', 'exit_without_confirmation', 'no')
            
            vomcfg.set('preferences', 'manage_secrets',
                       get_os_object().manage_secrets)
            vomcfg.write()
        
        link_configuration()
        
        conn = vomcfg.get('connection', 'connection')
        if not conn: # the user never never ran the app
            if self.device:
                self.sview.set_text(_('Running hardware wizard...'))
                self.sview.set_fraction(.2)
                    
                model = DeviceConfModel(self.device)
                ctrl = DeviceConfController(model, self.sview)
                view = DeviceConfView(ctrl)
                view.run()
        
        self.sview.set_fraction(.3)
        if self.device:
            self._init_sim()
            self.check_pin()
        else: # no device
            hook_it_up(self.sview, self.dev_listener, self.device)
        
    def check_pin_cb(self, result):
        resp = result[0].group('resp')
        self.sview.set_fraction(.4)
        if resp == '+CPIN: READY':
            self.sview.set_text(_('Authentication not necessary!'))
            self.sview.set_fraction(.5)
            hook_it_up(self.sview, self.dev_listener, self.device)
                
        elif resp == '+CPIN: SIM PIN':
            model = AskPINModel(self.device)
            ctrl = AskPINAndExecuteFuncController(model)
            ctrl.set_callback(hook_it_up, self.sview,
                              self.dev_listener, self.device)
            ctrl.set_errback(shutdown_core)
            ctrl.set_mode('send_pin')
            
            view = AskPINView(ctrl)
            view.show()
                    
        elif resp == '+CPIN: SIM PUK':
            model = AskPINModel(self.device)
            ctrl = AskPUKAndExecuteFuncController(model)
            ctrl.set_callback(hook_it_up, self.sview,
                              self.dev_listener, self.device)
            ctrl.set_errback(shutdown_core)
            ctrl.set_mode('send_puk')
            
            view = AskPUKView(ctrl)
            view.show()

    def sim_no_present(self, failure):
        failure.trap(ex.CMEErrorSIMNotInserted)
        hook_it_up(self.sview, self.dev_listener, None)
                
    def sim_busy_eb(self, failure):
        failure.trap(ex.CMEErrorSIMBusy, ex.CMEErrorSIMNotStarted,
                     ex.CMEErrorSIMFailure, ex.ATError)
        
        reactor.callLater(6, self.check_pin)
    
    def sim_failure_eb(self, failure):
        failure.trap(ex.CMEErrorSIMFailure)
        self.num_sim_failure_errors += 1
        if self.num_sim_failure_errors >= 3:
            hook_it_up(self.sview, self.dev_listener, None)
            return
        
        reactor.callLater(6, self.check_pin)
        
    def check_pin(self):
        self.sview.set_text(_('Authenticating...'))
        d = self.device.sconn.check_pin()
        d.addCallback(self.check_pin_cb)
        d.addErrback(self.sim_failure_eb)
        d.addErrback(self.sim_busy_eb)
        d.addErrback(self.sim_no_present)
    
class SerialService(Service):
    """
    I am a twistd Service that starts a GUI and hooks up a serial port
    """
    
    def __init__(self, device, debug):
        self.device = device
        self.debug = debug
    
    def startService(self):
        if self.device:
            sconn = SIMCardConnAdapter(self.device, self.debug)
            bdrate = self.device.baudrate
            cport = self.device.cport
            self.device.sconn = sconn
            SerialPort(sconn, cport, reactor, baudrate=bdrate)
        
        StartupController(self.device)
        
def attach_serial_protocol(device, debug=False, test=False):
    """
    Attachs the protocol to the reactor and returns the protocol's instance
    """
    if not test:
        sconn = SIMCardConnAdapter(device, debug)
    else:
        sconn = SIMCardConnection(device, debug)

    SerialPort(sconn, device.cport, reactor, baudrate=device.baudrate)
    return sconn

def hook_it_up(sview, device_listener, device=None):
    """Attachs comms core to GUI and presents main screen"""
    # switch to UCS2 as encoding
    def set_ucs2_charset_cb(_): pass
    def set_ucs2_charset_eb(failure):
        failure.trap(ex.CMEErrorOperationNotAllowed)
        log.err(failure)
    
    if device:
        device.sconn.set_character_set(
                "UCS2").addCallbacks(set_ucs2_charset_cb, set_ucs2_charset_eb)
    
    # get main screen up
    from vmc.models.application import ApplicationModel
    from vmc.views.application import ApplicationView
    from vmc.controllers.application import ApplicationController
    
    model = ApplicationModel(sview, device)
    ctrl = ApplicationController(model, device_listener)
    view = ApplicationView(ctrl)
    
    from vmc import louie
    from vmc.louie.plugin import TwistedDispatchPlugin
    louie.install_plugin(TwistedDispatchPlugin())

def create_service(device):
    """Returns a C{SerialService} instance"""
    debug = True
    return SerialService(device, debug)

# dependencies and permissions stuff

def check_dependencies():
    """checks that dependencies are satisfied"""
    resp = []
    try:
        import pygtk
        pygtk.require("2.0")
    except (ImportError, AssertionError):
        resp.append("python-gtk2 module not found, please install it")
        
    try:
        from twisted.copyright import version
        if [int(x) for x in re.search(r'^(\d+)\.(\d+)\.(\d+)',
                      version).groups()] < [ 2, 2, 0, ]:
            
            resp.append("python-twisted module is too old, please upgrade it")
        del version
        
    except ImportError:
        resp.append("python-twisted module not found, please install it")
    
    return resp

def check_permissions():
    array = [(mark_for_oal('PAP_SECRETS'), 0660),
             (mark_for_oal('CHAP_SECRETS'), 0660),
            ]
    
    response = []
    for elem in array:
        path, permissions = elem
        st = os.stat(path)
        mode = stat.S_IMODE(os.stat(path)[stat.ST_MODE])
        if permissions != mode:
            response.append("%s should have 0%03o mode, found 0%03o" % (path,
                                                        permissions, mode))
    
    # now check /etc/ppp/peers permissions
    peerspath = os.path.dirname(mark_for_oal('WVDIAL_AUTH_CONF'))
    if not os.access(peerspath, os.W_OK | os.R_OK):
        # the os.access call fails on a unionfs (Feisty Live)
        # opening a file in write mode in /etc/ppp/peers also fails, instead
        # we create a file in /tmp and try to copy it to /etc/ppp/peers if
        # that doesn't fails that means we're on a LiveCD with unionfs
        fobj = open('/tmp/foo.bar', 'w')
        finalpath = \
            os.path.join(os.path.dirname(mark_for_oal('WVDIAL_AUTH_CONF')),
                         'foo.bar')
        try:
            shutil.copy('/tmp/foo.bar', finalpath)
        except IOError, e:
            if e.errno == errno.EACCES:
                pass
            else:
                raise
        else:
            # oh you tricky unionfs
            # cleanup trash left
            os.unlink(finalpath)
            return response
        
        gid = os.stat(peerspath).st_gid
        groupinfo = grp.getgrgid(gid)
        msg = '%s should be readable and writtable by group %s' % (peerspath,
                                                                groupinfo[0])
        response.append(msg)
        uid = os.geteuid()
        userinfo = pwd.getpwuid(uid)
        username = userinfo[0]
        if username not in groupinfo.gr_mem:
            msg = 'user %s should be a member of group %s' % (username,
                                                              groupinfo[0]) 
            response.append(msg)
                       
    return response
    