# -*- coding: utf-8 -*-
# Copyright (C) 2006-2007  Vodafone España, S.A.
# Author:  Pablo Martí
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
I manage wvdial's profiles

To sum up, I'm a big ugly hack needed to overcome a wvdial/pppd deficiency.

According to wvdial's man page::
 /etc/ppp/peers/wvdial
     Required for correct authentication in pppd version 2.3.0 or newer.

Wvdial reads the authentication details from that file and only that file.
This is obviously not good in a multi-user environment, and we are left with
no choice but simlinking dynamically /etc/ppp/peers/wvdial to the given custom
config file. This is hacky and not really desirable, but we gotta stick to
this solution until we find a better one.
"""
__version__ = "$Rev: 634 $"

import os
import errno

import vmc.utils.globals
from vmc.common.config import VMCConfig
import vmc.common.exceptions as ex

class WvdialProfile(object):
    """I represent a custom wvdial profile"""
    def __init__(self, path):
        self.path = path
        self.name = os.path.basename(path)
        self._directives = [line.replace('\n', '') for line
                                in open(self.path).readlines()]
    
    def serialise(self):
        fobj = open(self.path, 'w')
        for directive in self._directives:
            fobj.write(directive + '\n')
        
        fobj.close()
    
    def has_key(self, key):
        if key in self._directives:
            return True
        return False
    
    def remove_key(self, key):
        if self.has_key(key):
            self._directives.remove(key)

def get_custom_profiles_list():
    """
    Returns a list with all profiles in $VMC_CFG/wvdial-configs
    """    
    return [WvdialProfile(os.path.join(vmc.utils.globals.WVDIAL_PROFILES,
                                       path))
                    for path in os.listdir(vmc.utils.globals.WVDIAL_PROFILES)]
    
def force_symlink(src, dst):
    """
    Creates a symlink between C{src} and C{dst}
    
    If C{dst} exists, it will be deleted.
    """
    try:
        os.symlink(src, dst)
    except OSError, e:
        if e.errno == errno.EEXIST:
            os.unlink(dst)
            os.symlink(src, dst)
        else:
            print "OSERROR", e
    
    except IOError, e:
        print "IOERROR", e

def link_configuration():
    """Symlinks the profile defined in the config to the actual file path"""
    conf = VMCConfig()
    custom = conf.getboolean('connection', 'use_custom_wvdial_profile')
    profile = custom and conf.get('connection',
                                  'wvdial_profile_name') or _('default')
    
    profile_path = os.path.join(vmc.utils.globals.WVDIAL_PROFILES, profile)
    
    force_symlink(profile_path, vmc.utils.globals.WVDIAL_AUTH_CONF)

def delete_profile(profile):
    """Deletes the given profile"""
    if not os.path.exists(profile.path):
        raise ex.ProfileNotFoundError
    
    try:
        os.unlink(profile.path)
    except (OSError, IOError):
        raise

def get_profile_from_name(name):
    """Returns a C{WvdialProfile} object corresponding to profile c{name}"""
    path = os.path.join(vmc.utils.globals.WVDIAL_PROFILES, name)
    return os.path.exists(path) and WvdialProfile(path) or None


from vmc.common.oal import mark_for_oal
from vmc.common.encoding import _
import shutil

def migrate_profiles():
    # between 0.9.6 wvdial profiles were renamed, this function migrates
    # them automagically, we will keep the old ones as .bak files in case
    # the user had something customised.
    default_migrated = False
    
    if 'predefinido' == _('default'):
        # this is only needed by Spanish users
        path = os.path.join(vmc.utils.globals.WVDIAL_PROFILES, 'default')
        shutil.move(path, path + '.bak')
        shutil.copy(os.path.join(vmc.utils.globals.EXTRA_DIR,
                             mark_for_oal('WVDIAL_DEFAULT_PROFILE')),
                vmc.utils.globals.DEFAULT_PROFILE)
        default_migrated = True
    
    for name in os.listdir(vmc.utils.globals.WVDIAL_PROFILES):
        if name in 'only-pap':
            path = os.path.join(vmc.utils.globals.WVDIAL_PROFILES, name)
            shutil.move(path, path + '.bak')
            shutil.copy(os.path.join(vmc.utils.globals.EXTRA_DIR,
                                     mark_for_oal('WVDIAL_PAP_PROFILE')),
                        vmc.utils.globals.PAP_PROFILE)
            
        elif name in 'only-chap':
            path = os.path.join(vmc.utils.globals.WVDIAL_PROFILES, name)
            shutil.move(path, path + '.bak')
            shutil.copy(os.path.join(vmc.utils.globals.EXTRA_DIR,
                                     mark_for_oal('WVDIAL_CHAP_PROFILE')),
                        vmc.utils.globals.CHAP_PROFILE)
    
    # now we have to check if after the changes we screwed the profile
    if not os.path.exists(vmc.utils.globals.WVDIAL_AUTH_CONF):
        name = os.path.basename(os.path.realpath(
                        vmc.utils.globals.WVDIAL_AUTH_CONF))
        
        if name in 'only-pap':
            os.unlink(vmc.utils.globals.WVDIAL_AUTH_CONF)
            os.symlink(vmc.utils.globals.WVDIAL_AUTH_CONF,
                       vmc.utils.globals.PAP_PROFILE)
        elif name in 'only-chap':
            os.unlink(vmc.utils.globals.WVDIAL_AUTH_CONF)
            os.symlink(vmc.utils.globals.WVDIAL_AUTH_CONF,
                       vmc.utils.globals.CHAP_PROFILE)
        
        if default_migrated and name in 'default':
            os.unlink(vmc.utils.globals.WVDIAL_AUTH_CONF)
            os.symlink(vmc.utils.globals.WVDIAL_AUTH_CONF,
                       vmc.utils.globals.DEFAULT_PROFILE)


def current_profile_uses_passwordplugin(self, conf=None):
    if not conf:
        conf = VMCConfig()
    
    custom = conf.getboolean('connection', 'use_custom_wvdial_profile')
    profile = custom and conf.get('connection',
                                  'wvdial_profile_name') or _('default')
    profile = get_profile_from_name(profile)
    if profile.has_key('plugin passwordfd.so'):
        return True
        
    return False
    