# -*- coding: utf-8 -*-
# Copyright (C) 2006-2007  Vodafone España, S.A.
# Author:  Pablo Martí
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""Controllers for the main window and the about dialog"""

__version__ = "$Rev: 658 $"

import os
import time
import ConfigParser

import gtk

from twisted.internet import reactor, task, defer
from twisted.python import log

from vmc.gtkmvc import Controller
from vmc.common.persistent import Contact
from vmc.common.config import VMCConfig
from vmc.common import wvdialprofile
from vmc.common.messages import Messages
from vmc.common.phonebook import PhoneBook
import vmc.common.exceptions as ex
from vmc.common.shutdown import shutdown_core
from vmc.common.oal import get_os_object
from vmc.common.notification import (show_error_notification,
                                 show_normal_notification)
from vmc.common.uptime import get_uptime_string
from vmc.common.netspeed import get_signal_level_from_rssi

from vmc.controllers.pin import (AskPINAndExecuteFuncController,
                             AskPUKAndExecuteFuncController)
from vmc.controllers.base import WidgetManager, TV_DICT, TV_DICT_REV

from vmc.models.pin import AskPINModel
from vmc.views.pin import AskPINView

import vmc.utils.dialogs
import vmc.utils.globals

from vmc.common.encoding import _

DEVICE_PRESENT, NO_DEVICE_PRESENT, DEVICE_ADDED = range(3)

class BaseApplicationController(Controller, WidgetManager):
    """Controller for the main window"""
    
    def __init__(self, model, device_listener):
        Controller.__init__(self, model)
        WidgetManager.__init__(self)
        self.device_listener = device_listener
        self.signal_loop = None
        self.net_loop = None
        self.time_loop = None
        self.start_time = 0
        self.status = None
        self.tray = None
        self.mode = None
        self._setup_trayicon()
    
    def quit_application(self, *args):
        """Closes open connections and exits the application"""
        # hide notifications just in case
        self.hide_widgets()
        title = _('Shutting down')
        apb = vmc.utils.dialogs.ActivityProgressBar(title, self)
        self.append_widget(apb)
        def default_cb():
            self.model.disconnect_internet()
            shutdown_core(delay=.3)
        def default_eb():
            pass
        
        apb.set_default_cb(2, default_cb)
        apb.set_cancel_cb(default_eb)
        apb.init()
    
    def register_view(self, view):
        """Overrides the register_view method and starts the whole thing up"""
        Controller.register_view(self, view)
        
        self.view.set_no_device_present()
        self.view.set_disconnected()
        
        self.view.setup_treeview(self)
        self.setup_signals()
        
        if self.model.device:
            self.mode = DEVICE_PRESENT
            self.start_device_activities()
        else:
            self.mode = NO_DEVICE_PRESENT
            self.go_to_last_setup()
    
    def setup_signals(self):
        self._setup_menubar_hacks()
        
        for treeview_name in list(set(TV_DICT.values())):
            treeview = self.view[treeview_name]
            treeview.connect('key_press_event', self.__on_treeview_key_press)
            if treeview.name != 'contacts_treeview':
                treeview.connect('row-activated', self._row_activated_tv)
        
        self.view.get_top_widget().connect("delete_event",
                                           self.quit_application)
        
    def start_device_activities(self, widget=None):
        if self.mode == NO_DEVICE_PRESENT:
            return
        
        def disable_callback(ignored):
            pass
        def disable_errback(failure):
            failure.trap(ex.CMEErrorOperationNotAllowed)
            log.err(failure, _(
"""
If you own a Huawei E220 device you can safely ignore this error, otherwise
send info your device and this traceback to the devs"""))
        self.model.disable_echo().addCallbacks(disable_callback,
                                                disable_errback)
        
        d = self.model.set_sms_indication(2, 1)
        def callback(ignored):
            self.model.set_sms_format(0)
            self.start_signal_quality_timer()
            self._fill_treeviews()
            d2 = self.model.get_pin_status()
            def callback2(active):
                if not active:
                    self.view['change_pin1_menu_item'].set_sensitive(False)
                    self.view['request_pin1'].set_active(False)
                if widget:
                    widget.close()
                    
            d2.addCallback(callback2)
        
        def pinreq_errback(failure):
            failure.trap(ex.CMEErrorSIMPINRequired)
            if widget:
                widget.close()
            model = AskPINModel(self.model.device)
            ctrl = AskPINAndExecuteFuncController(model)
            ctrl.set_callback(self.start_device_activities)
            ctrl.set_mode('send_pin')
            view = AskPINView(ctrl)
            view.show()
        
        def pukreq_errback(failure):
            failure.trap(ex.CMEErrorSIMPUKRequired)
            if widget:
                widget.close()
            model = AskPINModel(self.model.device)
            ctrl = AskPUKAndExecuteFuncController(model)
            ctrl.set_callback(self.start_device_activities)
            ctrl.set_mode('send_puk')
            view = AskPINView(ctrl)
            view.show()
            
        d.addCallback(callback)
        d.addErrback(pinreq_errback)
        d.addErrback(pukreq_errback)
    
    def _setup_menubar_hacks(self):
        def fake_delete_event(widget, event):
            if event.button == 1:
                self.on_delete_menu_item_activate(widget)
                return True
            
            return False
        
        def fake_forward_event(widget, event):
            if event.button == 1:
                self.on_forward_sms_menu_item_activate(widget)
                return True
            
            return False
        
        items = ['imagemenuitem8', 'sms_delete_menu_item',
                 'forward_sms_menu_item']
        
        for item in items:
            self.view[item].set_events(gtk.gdk.BUTTON_PRESS_MASK)
        
        # contacts_menubar delete item
        self.view['imagemenuitem8'].connect("button_press_event",
                                            fake_delete_event)
        # messages_menubar delete item
        self.view['sms_delete_menu_item'].connect("button_press_event",
                                                  fake_delete_event)
        # messages_menubar forward item
        self.view['forward_sms_menu_item'].connect("button_press_event",
                                                   fake_forward_event)
    
    def _empty_treeviews(self):
        for treeview_name in ['inbox_treeview', 'contacts_treeview']:
            self.view[treeview_name].get_model().clear()
    
    def _fill_treeviews(self):
        """Fills the treeviews with the SIM and HD's data
        
        Returns a deferred that will be callbacked when everything's ready
        """
        deferred = defer.Deferred()
        
        # fill sms
        self.model.set_splash_text(_('Reading messages...'))
        messages = Messages(self.model.get_sconn())
        
        def process_sms(sms_list):
            for sms in sms_list:
                active_tv = TV_DICT[sms.where]         # get treeview name
                treeview = self.view[active_tv]        # get treeview object
                treeview.get_model().add_message(sms)  # append sms to tv
                
            self.model.set_splash_fraction(.85)
            messages.close()
        
        messages.get_messages().addCallback(process_sms)
        
        # fill contacts
        self.model.set_splash_text(_('Reading contacts...'))
        phonebook = PhoneBook(self.model.get_sconn())

        def process_contacts(contacts):
            _model = self.view['contacts_treeview'].get_model()
            _model.add_contacts(contacts)
            self.model.set_splash_fraction(1)
            phonebook.close()
            self.go_to_last_setup(deferred)
        
        phonebook.get_contacts().addCallback(process_contacts)
    
    def go_to_last_setup(self, deferred=None):
        self._hide_splash_and_show_ourselves(deferred)
        if self.model.device:
            d = self._register_with_network()
            d.addCallback(self.view.set_device_present)
    
    def _register_with_network(self):
        self.set_cell_type()
        
        self.view.start_throbber()
        d = self.model.register_with_network()
        def callback(netinfo):
            if not netinfo:
                reactor.callLater(2, self._register_with_network)
                return
            
            netname, conn_type = netinfo
            self.view['network_name_label'].set_text(netname)
            self.view['cell_type_label'].set_text(conn_type)
            self.view.stop_throbber()
            return True
        
        d.addCallback(callback)
        
        self.view['network_name_label'].set_text(_('Associating...'))
        d2 = defer.Deferred()
        d.chainDeferred(d2)
        return d2
    
    def _hide_splash_and_show_ourselves(self, deferred=None):
        if deferred:
            deferred.callback(True)
        
        self.model.set_splash_fraction(1.0)
        # now we are done, hide the splash screen and show ourselves
        self.model.sview.hide()
        self.view.get_top_widget().show()
    
    #----------------------------------------------#
    # Signals Handling                             #
    #----------------------------------------------#
    
    def on_quit_menu_item_activate(self, widget):
        conf = VMCConfig()
        try:
            exit_without_conf = conf.getboolean('preferences',
                                            'exit_without_confirmation')
        except ConfigParser.NoOptionError:
            # as this option was introdused in 0.9.7, people with pre-0.9.7
            # software would get an exception here, this automigrates the
            # preferences file :)
            exit_without_conf = False
            conf.set('preferences',
                                  'exit_without_confirmation', 'no')
            conf.write()
            
        if exit_without_conf:
            conf.close()
            self.quit_application()
            return
        
        resp, checked = \
            vmc.utils.dialogs.open_dialog_question_checkbox_cancel_ok(
                    self.view,
                    _("Quit %s") % vmc.utils.globals.APP_LONG_NAME,
                    _("Are you sure you want to exit?"))
        
        val = checked and 'yes' or 'no'
        conf.set('preferences', 'exit_without_confirmation', val)
        conf.write()
        conf.close()
        
        if resp:
            self.quit_application()
    
    def on_diagnostics_item_activate(self, widget):
        from vmc.views.diagnostics import DiagnosticsView
        from vmc.controllers.diagnostics import DiagnosticsController
        from vmc.models.diagnostics import DiagnosticsModel
        
        model = DiagnosticsModel(self.model.device)
        ctrl = DiagnosticsController(model, self)
        view = DiagnosticsView(ctrl)
        view.set_parent_view(self.view)
        
        view.show()

    def on_about_menu_item_activate(self, widget):
        about = vmc.utils.dialogs.get_about_dialog()
        about.run()
        about.destroy()
    
    def on_change_pin1_menu_item_child_activate(self, widget):
        from vmc.views.pin import PinModifyView
        from vmc.controllers.pin import PinModifyController
        
        ctrl = PinModifyController(self.model)
        view = PinModifyView(ctrl)
        view.show()
    
    def on_request_pin1_activate(self, checkmenuitem):
        model = AskPINModel(self.model.device)
        
        active = checkmenuitem.get_active()
        if active:
            # The user wants the pin to be asked at startup
            # first we check if its already set
            d = self.model.get_pin_status()
            def callback(active):
                if active:
                    self.view['change_pin1_menu_item'].set_sensitive(True)
                    self.view['request_pin1'].set_active(True)
                else:
                    def func_callback(parent_ctrl):
                        parent_ctrl.view[
                                 'change_pin1_menu_item'].set_sensitive(True)
                        parent_ctrl.view['request_pin1'].set_active(True)
                    
                    def func_errback(parent_ctrl):
                        parent_ctrl.view['request_pin1'].set_active(False)
                    
                    ctrl = AskPINAndExecuteFuncController(model)
                    ctrl.set_callback(func_callback, self)
                    ctrl.set_errback(func_errback, self)
                    view = AskPINView(ctrl)                
                    ctrl.set_mode('enable_pin')
                    view.show()
                
            d.addCallback(callback)
            
        else:  # The user doesn't wants the pin to be asked
            d = self.model.get_pin_status()
            def callback(active):
                if active:
                    def func_callback(parent_ctrl):
                        parent_ctrl.view[
                                 'change_pin1_menu_item'].set_sensitive(False)
                        parent_ctrl.view['request_pin1'].set_active(False)
                    
                    def func_errback(parent_ctrl):
                        parent_ctrl.view['request_pin1'].set_active(True)
                    
                    ctrl = AskPINAndExecuteFuncController(model)  
                    ctrl.set_callback(func_callback, self)
                    ctrl.set_errback(func_errback, self)
                    view = AskPINView(ctrl)                
                    ctrl.set_mode('disable_pin')
                    view.show()
                else:
                    self.view['change_pin1_menu_item'].set_sensitive(False)
                    self.view['request_pin1'].set_active(False)
            
            d.addCallback(callback)
            
    def on_profiles_menu_item_activate(self, widget):
        raise NotImplementedError
    
    def on_updates_menu_item_activate(self, widget):
        raise NotImplementedError
    
    def on_preferences_menu_item_activate(self, widget):
        from vmc.views.preferences import PreferencesView
        from vmc.models.preferences import PreferencesModel
        from vmc.controllers.preferences import PreferencesController
        
        model = PreferencesModel(self.model.device)
        ctrl = PreferencesController(model, self)
        view = PreferencesView(ctrl, self.model.device)
        view.show()
    
    def on_mobile1_activate(self, widget):
        # This emits the toggled signal
        self.view['connect_button'].set_active(True)
    
    def on_import_contacts1_activate(self, widget):
        filepath = vmc.utils.dialogs.open_import_csv_dialog()
        if filepath:
            from vmc.common.csvutils import CSVUnicodeReader
            reader = CSVUnicodeReader(open(filepath))
            _model = self.view['contacts_treeview'].get_model()
            
            phonebook = PhoneBook(self.model.get_sconn())
            def add_it(contact):
                _model.add_contact(contact)
                phonebook.close()
            
            def callback(free):
                for row in reader:
                    name, number = row
                    number = number.encode('utf-8').replace('"', '').strip()
                    index = free.popleft()
                    contact = Contact(name, number, index)
                    # add them to the SIM
                    phonebook.add_contact(contact, True).addCallback(add_it)
            
            d = self.model.get_free_contact_ids()
            d.addCallback(callback)
                
    def on_export_contacts1_activate(self, widget):
        filepath = vmc.utils.dialogs.save_csv_file()
        if filepath:
            from vmc.common.csvutils import CSVUnicodeWriter
            writer = CSVUnicodeWriter(open(filepath, 'w'))
            phonebook = PhoneBook(self.model.get_sconn())
            def callback(contacts):
                for contact in contacts:
                    writer.write_row(contact.to_csv())
                phonebook.close()
            
            phonebook.get_contacts().addCallback(callback)
            
    def on_connect_button_toggled(self, button):
        if button.get_active():
            # check if its necessary to manage pppd's secrets
            if get_os_object().manage_secrets:
                # check if all the profile stuff is cool
                conf = VMCConfig()
                if wvdialprofile.current_profile_uses_passwordplugin(conf):
                    message = _("WVDial directive error")
                    details = _("""
Since version 0.9.7 %s manages pppd's /etc/ppp/{p,ch}ap-secrets, the
'plugin passwordfd.so' directive should not appear on the
current wvdial profile, please remove it manually. Go to
Tools -> Preferences, select the wvdial profiles tab, and
remove from the current wvdial profile the
'plugin passwordfd.so' line.""") % vmc.utils.globals.APP_LONG_NAME
                    vmc.utils.dialogs.open_warning_dialog(message, details)
                    self.view['connect_button'].set_active(False)
                    conf.close()
                    return
                
                conf.close()
            
            # the button was not active and now is active
            title = _('Connecting...')
            apb = vmc.utils.dialogs.ActivityProgressBar(title, self)
            self.append_widget(apb)
            d = self.model.connect_internet()
            def callback(resp):
                apb.close()
                self.start_network_stats_timer()
                
                d = self._register_with_network()
                def callback2(ignored):
                    self.view.set_connected()
                
                    d = self.model.get_network_info()
                    def callback3(netinfo):
                        self.view['net_statusbar'].pop(1)
                        if netinfo:
                            msg = _('Connected to %s') % netinfo[1]
                            self.view['net_statusbar'].push(1, msg)
                        
                    d.addCallback(callback3)
                d.addCallback(callback2)
            
            d.addCallbacks(callback, lambda failure: log.err(failure))
            self.model.connecting = True
            self.view['net_statusbar'].push(1, _('Connecting...'))
            def default_eb():
                self.model.disconnect_internet()
                self.view.set_disconnected()
            
            apb.set_cancel_cb(default_eb)
            apb.init()
        else:
            # the button was active and now it isn't
            self.model.disconnect_internet()
            self.model.connecting = False
            self.model.internet_connected = False
            self.view['mobile1'].set_sensitive(True)
            self.view['preferences_menu_item'].set_sensitive(True)
            self.stop_network_stats_timer()
                
            self.view.set_disconnected()
            
    def on_sms_menu_item_activate(self, widget):
        self.on_sms_button_clicked(None)
    
    def on_usage_menu_item_activate(self, widget):
        self.on_usage_button_clicked(None)
    
    def on_support_menu_item_activate(self, widget):
        self.on_support_button_clicked(None)
    
    def on_minimize_menu_item_activate(self, widget):
        pass
    
    def on_sms_preferences_activate(self, widget):
        from vmc.controllers.preferences import SMSPreferencesController
        from vmc.models.preferences import PreferencesModel
        from vmc.views.preferences import SMSPreferencesView
        
        model = PreferencesModel(self.model.device)
        ctrl = SMSPreferencesController(model)
        view = SMSPreferencesView(ctrl, self)
        view.set_parent_view(self.view)
        view.show()
        
    def on_mail_button_clicked(self, widget):
        if not self.__check_if_connected():
            return
        
        from vmc.common.config import get_default_binaries
        bindict = get_default_binaries()
        from twisted.internet.utils import getProcessOutput
        getProcessOutput(bindict['mua'], [], os.environ)
    
    def on_sms_button_clicked(self, widget):
        if not self.model.sms_shown:
            if self.model.usage_shown:
                self.view['usage_frame'].hide()
                self.model.usage_shown = False
            elif self.model.support_shown:
                self.view['support_notebook'].hide()
                self.model.support_shown = False
            self.view['sms_frame_alignment'].show()
            self.model.sms_shown = True
        
    def on_internet_button_clicked(self, widget):
        if not self.__check_if_connected():
            return
        
        from vmc.common.config import get_default_binaries
        bindict = get_default_binaries()
        from twisted.internet.utils import getProcessOutput
        getProcessOutput(bindict['browser'], [], os.environ)
        
    def on_usage_button_clicked(self, widget):
        if not self.model.usage_shown:
            if self.model.sms_shown:
                self.view['sms_frame_alignment'].hide()
                self.model.sms_shown = False
            if self.model.support_shown:
                self.view['support_notebook'].hide()
                self.model.support_shown = False
            self.view['usage_frame'].show()
            self.model.usage_shown = True
    
    def on_support_button_clicked(self, widget):
        if not self.model.support_shown:
            if self.model.usage_shown:
                self.view['usage_frame'].hide()
                self.model.usage_shown = False
            elif self.model.sms_shown:
                self.view['sms_frame_alignment'].hide()
                self.model.sms_shown = False
            self.view['support_notebook'].show()
            self.model.support_shown = True
    
    def on_new_contact_menu_item_activate(self, widget):
        from vmc.views.contacts import AddContactView
        from vmc.controllers.contacts import AddContactController
        self.view['main_notebook'].set_current_page(3) # contacts_tv
        ctrl = AddContactController(self.model, self)
        view = AddContactView(ctrl)
        view.set_parent_view(self.view)
        view.show()
    
    def on_search_contact_menu_item_activate(self, widget):
        from vmc.views.contacts import SearchContactView
        from vmc.controllers.contacts import SearchContactController
        self.view['main_notebook'].set_current_page(3) # contacts_tv
        ctrl = SearchContactController(self.model, self)
        view = SearchContactView(ctrl)
        view.run()
    
    def on_new_sms_activate(self, widget):
        from vmc.views.sms import NewSmsView
        from vmc.models.sms import NewSmsModel
        from vmc.controllers.sms import NewSmsController
        
        ctrl = NewSmsController(NewSmsModel(), self)
        view = NewSmsView(ctrl)
        view.set_parent_view(self.view)
        
        view.show()
    
    def on_reply_sms_no_quoting_menu_item_activate(self, widget):
        message = self.get_obj_from_selected_row()
        if not message:
            return
        
        from vmc.views.sms import ForwardSmsView
        from vmc.models.sms import NewSmsModel
        from vmc.controllers.sms import ForwardSmsController
        
        ctrl = ForwardSmsController(NewSmsModel(), self)
        view = ForwardSmsView(ctrl)
        view.set_parent_view(self.view)
        
        ctrl.set_recipient_numbers(message.get_number())
        view.show()
    
    def on_reply_sms_quoting_menu_item_activate(self, widget):
        message = self.get_obj_from_selected_row()
        if not message:
            return
        
        from vmc.views.sms import ForwardSmsView
        from vmc.models.sms import NewSmsModel
        from vmc.controllers.sms import ForwardSmsController
        
        ctrl = ForwardSmsController(NewSmsModel(), self)
        view = ForwardSmsView(ctrl)
        ctrl.set_textbuffer_text(message.get_text())
        ctrl.set_recipient_numbers(message.get_number())
        view.set_parent_view(self.view)
        view.show()
    
    def on_forward_sms_menu_item_activate(self, widget):
        message = self.get_obj_from_selected_row()
        if not message:
            return
        
        from vmc.views.sms import ForwardSmsView
        from vmc.models.sms import NewSmsModel
        from vmc.controllers.sms import ForwardSmsController
        
        ctrl = ForwardSmsController(NewSmsModel(), self)
        view = ForwardSmsView(ctrl)
        ctrl.numbers_entry.grab_focus()
        ctrl.set_textbuffer_text(message.get_text())
        view.set_parent_view(self.view)
        view.show()
    
    def on_add_contact_menu_item_activate(self, widget):
        self.on_new_contact_menu_item_activate(None)
    
    def on_delete_menu_item_activate(self, widget):
        page = self.view['main_notebook'].get_current_page() + 1
        treeview = self.view[TV_DICT[page]]
        self.delete_entries(widget, None, treeview)
        treeview.grab_focus()
    
    def on_generic_treeview_row_button_press_event(self, treeview, event):
        if treeview.name in ['inbox_treeview', 'outbox_treeview',
                                 'drafts_treeview']:
            
            if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
                pass
            elif event.button == 3 and event.type == gtk.gdk.BUTTON_PRESS:
                selection = treeview.get_selection()
                model, pathlist = selection.get_selected_rows()
                if pathlist:
                    menu = self.get_generic_popup_menu(pathlist, treeview)
                    menu.popup(None, None, None, event.button, event.time)
                    return True # selection is not lost
                    
        else: # contacts_treeview
            if event.button == 1 and event.type == gtk.gdk.BUTTON_PRESS:
                pass
            elif event.button == 3 and event.type == gtk.gdk.BUTTON_PRESS:
                selection = treeview.get_selection()
                model, pathlist = selection.get_selected_rows()
                if pathlist:
                    menu = self.get_contacts_popup_menu(pathlist, treeview)
                    menu.popup(None, None, None, event.button, event.time)
                    return True # selection is not lost
    
    def on_cursor_changed_treeview_event(self, treeview):
        col = treeview.get_cursor()[0]
        model = treeview.get_model()
        text = model[col][1]
        buffer = self.view['smsbody_textview'].get_buffer()
        buffer.set_text(text)
        self.view['vbox17'].show()

    def on_main_notebook_switch_page(self, notebook, ptr, pagenum):
        pagenum = int(pagenum)
        if pagenum == 3:
            self.view['contacts_menubar'].show()
            self.view['sms_menubar'].hide()
        else:
            self.view['contacts_menubar'].hide()
            self.view['sms_menubar'].show()
        
        self.view['vbox17'].hide()
            
    #----------------------------------------------#
    # MISC FUNCTIONALITY                           #
    #----------------------------------------------#
    
    def _name_contact_cell_edited(self, widget, path, newvalue):
        """Handler for the cell-edited signal of the name column"""
        model = self.view['contacts_treeview'].get_model()
        # first check that the edit is necessary
        if newvalue == model[path][1]:
            return
        
        model[path][1] = newvalue
        contact = model[path][3]
        contact.name = newvalue
        model[path][3] = contact
        
        phonebook = PhoneBook(self.model.get_sconn())
        callback = lambda _: phonebook.close()
        phonebook.edit_contact(contact).addCallback(callback)
        
    def _number_contact_cell_edited(self, widget, path, newvalue):
        """Handler for the cell-edited signal of the number column"""
        model = self.view['contacts_treeview'].get_model()
        number = newvalue.strip()
        # first check that the edit is necessary
        if number == model[path][2]:
            return
        
        from vmc.utils.utilities import is_valid_number
        if not is_valid_number(number):
            return
        
        contact = model[path][3]
        contact.number = newvalue
        model[path][2] = number
        model[path][3] = contact
        
        phonebook = PhoneBook(self.model.get_sconn())
        callback = lambda _: phonebook.close()
        phonebook.edit_contact(contact).addCallback(callback)
        
    def _setup_trayicon(self):
        from vmc.common.tray import get_tray_icon
        self.tray = get_tray_icon(self._show_hide_window,
                                  self.get_trayicon_popup_menu)
    
    def _show_hide_window(self, *args):
        if len(args) == 1:
            # we ask the for the number of args because we use this
            # function for both egg.tray.TrayIcon and gtk.StatusIcon
            # the formers callback only has one arg
            window = self.view.get_top_widget()
            if window.get_property('visible'):
                window.hide()
            else:
                window.present()
        else:
            eventbox, event = args
            if event.button == 1: # left click
                window = self.view.get_top_widget()
                if window.get_property('visible'):
                    window.hide()
                else:
                    window.present()
            elif event.button == 3: # right click
                menu = self.get_trayicon_popup_menu()
                menu.popup(None, None, None, event.button, event.time)
    
    def _row_activated_tv(self, treeview, path, col):
        # get selected row
        model, selected = treeview.get_selection().get_selected_rows()
        if len(path) > 1: # more than one? no way
            return
        
        iter_ = model.get_iter(selected[0])
        text = model.get_value(iter_, 1)
        number = model.get_value(iter_, 2)
        
        from vmc.views.sms import ForwardSmsView
        from vmc.models.sms import NewSmsModel
        from vmc.controllers.sms import ForwardSmsController
        
        ctrl = ForwardSmsController(NewSmsModel(), self)
        view = ForwardSmsView(ctrl)
        ctrl.set_textbuffer_text(text)
        ctrl.set_recipient_numbers(number)
        view.show()
    
    def __check_if_connected(self):
        """
        Returns True if connected or the user doesn't cares is not connected
        """
        if self.model.internet_connected:
            return True
        else:
            message = _("Not connected")
            details = _("No mobile connection. Do you want to continue?")
            return vmc.utils.dialogs.open_warning_request_cancel_ok(message,
                                                        details)
    
    def __on_treeview_key_press(self, widget, event):
        """Handler for key_press_button in treeviews"""
        if event.keyval == 65535: # Key Del was pressed 
            # get current treeview
            num = self.view['main_notebook'].get_current_page() + 1
            treeview_name = TV_DICT[num]
            treeview = self.view[treeview_name]
            self.delete_entries(None, None, treeview)
    
    def delete_entries(self, menuitem, pathlist, treeview):
        """Deletes the entries selected in the treeview
        
        This entries are deleted in SIM/HD and the treeview"""
        model, selected = treeview.get_selection().get_selected_rows()
        iters = [model.get_iter(path) for path in selected]
        pos = treeview.name in 'contacts_treeview' and 3 or 4
        
        objs_to_del = [model.get_value(_iter, pos) for _iter in iters]
        
        if treeview.name in 'contacts_treeview':
            phonebook = PhoneBook(self.model.get_sconn())
            d = phonebook.delete_contacts(objs_to_del)
            d.addCallback(lambda _: phonebook.close())
        else:
            messages = Messages(self.model.get_sconn())
            d = messages.delete_messages(objs_to_del)
            d.addCallback(lambda _: messages.close())
        
        for _iter in iters:
            model.remove(_iter) # delete from treeview
        
        # If we are in a sms treeview and no sms is selected, clear textview
        if treeview.get_name() != 'contacts_treeview':
            obj = self.get_obj_from_selected_row()
            if not obj:
                buffer = self.view['smsbody_textview'].get_buffer()
                buffer.set_text('')
    
    def _send_sms_to_contact(self, menuitem, treeview):
        selection = treeview.get_selection()
        model, selected = selection.get_selected_rows()
        iters = [model.get_iter(path) for path in selected]
        numbers = [model.get_value(_iter, 2) for _iter in iters]
        
        from vmc.views.sms import NewSmsView
        from vmc.models.sms import NewSmsModel
        from vmc.controllers.sms import NewSmsController
        
        ctrl = NewSmsController(NewSmsModel(), self)
        view = NewSmsView(ctrl)
        view.set_parent_view(self.view)
        view.show()
        
        ctrl.set_entry_text(", ".join(numbers))
    
    def get_trayicon_popup_menu(self, *args):
        """Returns a popup menu when you right click on the trayicon"""
        def __disconnect_from_inet(widget):
            button = gtk.ToggleButton()
            button.set_active(False)
            self.on_connect_button_toggled(button)
        
        def __connect_to_inet(widget):
            button = gtk.ToggleButton()
            button.set_active(True)
            self.on_connect_button_toggled(button)
        
        menu = gtk.Menu()
        
        if self.model.internet_connected:
            item = gtk.ImageMenuItem(_("Disconnect"))
            img = gtk.Image()
            img.set_from_file(os.path.join(vmc.utils.globals.IMAGES_DIR,
                              'stop16x16.png'))
            item.set_image(img)
            item.connect("activate", __disconnect_from_inet)
            item.show()
            menu.append(item)
        
        else:
            item = gtk.ImageMenuItem(_("Connect"))
            img = gtk.Image()
            img.set_from_file(os.path.join(vmc.utils.globals.IMAGES_DIR,
                              'connect-16x16.png'))
            item.set_image(img)
            item.connect("activate", __connect_to_inet)
            item.show()
            menu.append(item)
        
        separator = gtk.SeparatorMenuItem()
        separator.show()
        menu.append(separator)
        
        item = gtk.ImageMenuItem(_("Send SMS"))
        img = gtk.Image()
        img.set_from_file(os.path.join(vmc.utils.globals.IMAGES_DIR,
                              'sms16x16.png'))
        item.set_image(img)
        item.connect("activate", self.on_new_sms_activate)
        item.show()
        menu.append(item)
        
        separator = gtk.SeparatorMenuItem()
        separator.show()
        menu.append(separator)
        
        item = gtk.ImageMenuItem(_("Quit"))
        img = gtk.image_new_from_stock(gtk.STOCK_QUIT, gtk.ICON_SIZE_MENU)
        item.set_image(img)
        item.connect("activate", self.on_quit_menu_item_activate)
        item.show()
        menu.append(item)
        
        return menu
    
    def get_contacts_popup_menu(self, pathinfo, treeview):
        """Returns a popup menu for the contacts treeview"""
        menu = gtk.Menu()
        item = gtk.MenuItem(_("_Send SMS"))
        item.connect("activate",
                     self._send_sms_to_contact, treeview)
        item.show()
        menu.append(item)
        
        item = gtk.ImageMenuItem(_("_Delete"))
        img = gtk.image_new_from_stock(gtk.STOCK_DELETE, gtk.ICON_SIZE_MENU)
        item.set_image(img)
        item.connect("activate", self.delete_entries, pathinfo, treeview)
        item.show()
        menu.append(item)
        
        return menu
    
    def get_generic_popup_menu(self, pathinfo, treeview):
        """Returns a popup menu for the rest of treeviews"""
        detailmenu = gtk.Menu()
        item = gtk.ImageMenuItem(_("_Add to contacts"))
        img = gtk.image_new_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_MENU)
        item.set_image(img)
        item.connect("activate", self._use_detail_add_contact)
        item.show()
        
        detailmenu.append(item)
        detailitem = gtk.MenuItem(_("_Use detail"))
        detailitem.set_submenu(detailmenu)
        detailitem.show()
        
        menu = gtk.Menu() # main menu
        menu.append(detailitem)
        
        if treeview.get_name() != 'drafts_treeview':
            item = gtk.ImageMenuItem(_("Save to draft"))
            img = gtk.image_new_from_stock(gtk.STOCK_SAVE, gtk.ICON_SIZE_MENU)
            item.set_image(img)
            item.connect("activate", self._save_sms_to_draft)
            item.show()
            menu.append(item)
            
        separator = gtk.SeparatorMenuItem()
        separator.show()
        menu.append(separator)
        
        item = gtk.ImageMenuItem(_("Delete"))
        img = gtk.image_new_from_stock(gtk.STOCK_DELETE, gtk.ICON_SIZE_MENU)
        item.set_image(img)
        item.connect("activate", self.delete_entries, pathinfo, treeview)
        item.show()
        menu.append(item)
        
        return menu
    
    def _save_sms_to_draft(self, widget):
        """This will save the selected SMS to the drafts tv and the DB"""
        message = self.get_obj_from_selected_row()
        if not message:
            return
        
        messages = Messages(self.model.get_sconn())
        def callback(sms):
            # Now save SMS to DB
            def callback2(smsrep):
                self.view['drafts_treeview'].get_model().add_message(smsrep)
                messages.close()
            
            messages.add_message(sms).addCallback(callback2)
        messages.get_message(message).addCallback(callback)
        
    def _use_detail_add_contact(self, widget):
        """Handler for the use detail menu"""
        message = self.get_obj_from_selected_row()
        if not message:
            return
        
        from vmc.views.contacts import AddContactView
        from vmc.controllers.contacts import AddContactController
        ctrl = AddContactController(self.model, self)
        view = AddContactView(ctrl)
        view.set_parent_view(self.view)
        ctrl.number_entry.set_text(message.get_number())
        view.show()
    
    def get_obj_from_selected_row(self):
        """Returns the data from the selected row"""
        page = self.view['main_notebook'].get_current_page() + 1
        
        treeview = self.view[TV_DICT[page]]
        selection = treeview.get_selection()
        model, selected = selection.get_selected_rows()
        if not selected or len(selected) > 1:
            return None
        
        _iter = model.get_iter(selected[0])
        row = (page == TV_DICT_REV['contacts_treeview']) and 3 or 4
        return model.get_value(_iter, row)
    
    def set_cell_type(self):
        """Sets the connection type in cell_type_label"""
        d = self.model.get_network_info()
        def callback(netinfo):
            if netinfo:
                netname, conn_type = netinfo
                if not netname or not conn_type:
                    # WTF  buggy hardware case, but may happen
                    reactor.callLater(4, self.set_cell_type)
                    return
                self.view['cell_type_label'].set_text(conn_type)
            else:
                reactor.callLater(3, self.set_cell_type)
        
        d.addCallback(callback)
    
    def set_network_name(self):
        """Sets the network name in network_name_label"""
        d = self.model.get_network_info()
        def callback(netinfo):
            if netinfo:
                netname, conn_type = netinfo
                if not netname or not conn_type:
                    # WTF buggy hardware case, but may happen
                    reactor.callLater(4, self.set_network_name)
                    return
                
                self.view['network_name_label'].set_text(netname)
                self.view['cell_type_label'].set_text(conn_type)
            else:
                reactor.callLater(4, self.set_network_name)
        
        d.addCallback(callback)
    
    def start_signal_quality_timer(self):
        """Starts the timer that regularly checks the signal quality"""
        self.signal_loop = task.LoopingCall(self.change_signal_level)
        self.signal_loop.start(20)
    
    def start_network_stats_timer(self):
        """Starts the timer that updates the speed stats on screen"""
        self.net_loop = task.LoopingCall(self.change_net_stats)
        self.net_loop.start(1)
    
    def start_connection_timer(self):
        self.start_time = time.time()
        self.time_loop = task.LoopingCall(self.change_time)
        self.time_loop.start(1)
    
    def stop_connection_timer(self):
        """Stops the connection timer"""
        self.time_loop.stop()
        self.view['time_label'].set_text('')
    
    def stop_network_stats_timer(self):
        """Stops the network stats timer"""
        try:
            self.net_loop.stop()
        except:
            pass
    
    def stop_signal_quality_timer(self):
        """Stops the signal quality stats timer"""
        try:
            self.signal_loop.stop()
        except:
            pass
    
    def change_time(self):
        """Changes the time on the time_label"""
        diff = time.time() - self.start_time
        conn_time = get_uptime_string(diff)
        self.view['time_label'].set_text(conn_time)
    
    def change_signal_level(self):
        """Changes the signal level of the bottom_progressbar"""
        def callback(slevel):
            rssi = int(slevel[0].group('rssi'))
            image = 'signal-%d.png' % get_signal_level_from_rssi(rssi)
            self.view['signal_image'].set_from_file(
                   os.path.join(vmc.utils.globals.IMAGES_DIR, image))
        
        d = self.model.get_signal_level()
        d.addCallback(callback)
    
    def change_net_stats(self):
        """Updates network statistics"""
        up = self.model.netspeed['up']
        down = self.model.netspeed['down']
        
        upspeed = (up * 8) / 1000.0
        downspeed = (down * 8) / 1000.0
        
        if upspeed > 1000:
            self.view['upload_statusbar'].push(1,
                                         "%3.2f Mbps" % (upspeed / 1000))
        else:
            self.view['upload_statusbar'].push(1, "%3.2f Kbps" % upspeed)
        
        if downspeed > 1000:
            self.view['download_statusbar'].push(1,
                                         "%3.2f Mbps" % (downspeed / 1000))
        else:
            self.view['download_statusbar'].push(1, "%3.2f Kbps" % downspeed)

class ApplicationController(BaseApplicationController):
    
    def __init__(self, model, device_listener):
        BaseApplicationController.__init__(self, model, device_listener)
        self._setup_louie_signals()
    
    def _setup_louie_signals(self):
        # louie signals
        from vmc import louie
        louie.connect(self.sms_recv, vmc.utils.globals.SIG_SMSRECV)
        louie.connect(self.timeout_handler, vmc.utils.globals.SIG_TIMEOUT)
        louie.connect(self.invalid_dns_handler,
                      vmc.utils.globals.SIG_INVALID_DNS)
        louie.connect(self.disconnected_handler,
                      vmc.utils.globals.SIG_DISCONNECTED)
        louie.connect(self.device_removed_handler,
                      vmc.utils.globals.SIG_DEVICE_REMOVED)
        louie.connect(self.device_added_handler,
                      vmc.utils.globals.SIG_DEVICE_ADDED)
    
    #################################################################
    # SIGNAL HANDLERS                                               #
    #################################################################
        
    def sms_recv(self, smsinfo):
        """I'm called by a louie's signal whenever a SMS is received"""
        assert smsinfo.group('where') == "SM"
        def callback(sms):
            self.view['inbox_treeview'].get_model().add_message(sms)
            
            phonebook = PhoneBook(self.model.get_sconn())
            def lookup_number_cb(contacts):
                if contacts:
                    who = contacts[0].name
                else:
                    who = sms.get_number()
                message = _("<small>SMS received from %s</small>") % who
                details = _(
                    "<small>Received at %(date)s</small>\n<b>%(text)s</b>") % \
                    {'date' : sms.get_localised_date(),
                     'text' : sms.get_text()}
                
                noti = show_normal_notification(self.tray, message, details)
                self.append_widget(noti)
                phonebook.close()
            
            phonebook.find_contact(
                    number=sms.get_number()).addCallback(lookup_number_cb)
            
        index = int(smsinfo.group('id'))
        self.model.get_sms(index).addCallback(callback)
    
    def timeout_handler(self, *args):
        """
        I'm called by a louie's signal whenever a timeout in an ATCmd occurs
        """
        message = _("Problems with SIM card connection")
        details = _(
"""
An unexpected internal problem has occured, this probably
has happened because the SIM card took too much to reply
my last command. Please eject and insert the card and start
%s again.""") % vmc.utils.globals.APP_LONG_NAME
        vmc.utils.dialogs.open_warning_dialog(message, details)
        reactor.stop()
    
    def invalid_dns_handler(self, dnslist):
        """Handler called when wvdial recibes invalid DNS"""
        title = _("Invalid DNS recibed from APN")
        message = _(
"""
<small>The DNS recibed from the APN are invalid. You will probably want
to restart the connection or if the problem does persist, you can specify
an static DNS entry in the connection preferences</small>
""")
        notification = show_error_notification(self.tray, title, message)
        self.append_widget(notification)
    
    def disconnected_handler(self, *args):
        """Handler called when wvdial dies before connecting"""
        if self.model.internet_connected:
            log.err("disconnected_handler this shouldn't happen")
            return
        
        self.hide_widgets()
        title = _("Disconnected from Internet")
        message = _(
"""
<small>%s has given
up after trying to connect three times to Internet. This might
be provoked by a problem in the configuration or just the fact
that there's no carrier.</small>""") % vmc.utils.globals.APP_LONG_NAME

        notification = show_error_notification(self.tray, title, message)
        self.append_widget(notification)
        self.view['connect_button'].set_active(False)
    
    def device_removed_handler(self):
        """Handler called when wvdial recibes invalid DNS"""
        message = _("Device in use removed")
        details = _(
"""
The 3G device in use has been removed. Now
%s
is going to shutdown. In order to continue,
plug in the 3G device and start again.""") % vmc.utils.globals.APP_LONG_NAME

        notification = show_error_notification(self.tray, message, details)
        
        self.hide_widgets()
        self.append_widget(notification)
        
        self.view.set_no_device_present()
        self.mode = NO_DEVICE_PRESENT
        self.model.disconnect_internet()
        
        self.stop_network_stats_timer()
        self.stop_signal_quality_timer()
        
        # clean treeviews
        self._empty_treeviews()
    
    def device_added_handler(self, device):
        self.model.device = device
        self.mode = DEVICE_ADDED
        
        from vmc.common.startup import attach_serial_protocol
        self.model.device.sconn = attach_serial_protocol(self.model.device,
                                                         True)
        config = VMCConfig()
        conn = config.get('connection', 'connection')
        config.close()
        
        if not conn:
            from vmc.controllers.initialconf import DeviceConfController
            from vmc.models.initialconf import DeviceConfModel
            from vmc.views.initialconf import DeviceConfView
            
            model = DeviceConfModel(device)
            ctrl = DeviceConfController(model)
            view = DeviceConfView(ctrl)
            view.run()
        
        self.hide_widgets()
        
        message = _("3G Device added")
        details = _(
"""
A new 3G device (%(device_name)s)
has been added, in around 15 seconds
(%(app_name)s) will resume its functioning""") % \
                {'device_name' : device.__name__,
                 'app_name' : vmc.utils.globals.APP_LONG_NAME}

        notification = show_normal_notification(self.tray, message, details)
        self.append_widget(notification)
        
        title = _("Setting up device...")
        apb = vmc.utils.dialogs.ActivityProgressBar(title, self,
                                                    initnow=True,
                                                    disable_cancel=True)
        self.append_widget(apb)
        reactor.callLater(15, self.start_device_activities, apb)
