# -*- coding: utf-8 -*-
# Copyright (C) 2006-2007  Vodafone España, S.A.
# Author:  Pablo Martí
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fi1fth Floor, Boston, MA 02110-1301 USA.
"""
Controllers for diagnostics
"""
__version__ = "$Rev: 652 $"

from vmc.gtkmvc import Controller
from vmc.common.encoding import _

class DiagnosticsController(Controller):
    """Controller for the diagnostics window"""
    
    def __init__(self, model, parent_ctrl):
        Controller.__init__(self, model)
        self.parent_ctrl = parent_ctrl
    
    def register_view(self, view):
        """
        Fill the label fields of the diagnostics dialog
        
        This will be called once the view is registered
        """
        Controller.register_view(self, view)
        
        if self.model.device:
            self.set_sim_info()
        
        self.model.get_uptime().addCallback(lambda uptime:
                self.view['uptime_number_label'].set_text(uptime))
        
        self.view['os_name_label'].set_text(self.model.get_os_name())
        self.view['os_version_label'].set_text(self.model.get_os_version())
        
        if not self.parent_ctrl.model.internet_connected:
            self.view['start_test_button'].set_sensitive(False)
    
    def set_sim_info(self):
        self.model.get_imei_number().addCallback(
                lambda imei: self.view['imei_number_label'].set_text(imei))
        self.model.get_imsi_number().addCallback(
                lambda imsi: self.view['imsi_number_label'].set_text(imsi))
        self.model.get_card_version().addCallback(
                lambda card_v: self.view['firmware_label'].set_text(card_v))
        self.model.get_card_model().addCallback(
                lambda card_m: self.view['card_model_label'].set_text(card_m))
    
    # ------------------------------------------------------------ #
    #                       Signals Handling                       #
    # ------------------------------------------------------------ #
    
    def on_report_button_clicked(self, widget):
        d = self.model.get_support_report()
        def report_callback(text):
            import vmc.utils.dialogs
            from vmc.utils.utilities import save_file
            path = vmc.utils.dialogs.save_standard_file()
            if path:
                save_file(path, text)
            self._hide_myself()
        
        d.addCallback(report_callback)
    
    def on_start_test_button_clicked(self, widget):
        from vmc.views.diagnostics import ConnectivityTestView
        ctrl = ConnectivityTestController(self.model, self)
        view = ConnectivityTestView(ctrl)
        view.set_parent_view(self.view)
        view.show()
    
    def on_close_button_clicked(self, widget):
        self._hide_myself()
    
    def _hide_myself(self):
        self.model.unregister_observer(self)
        self.view.hide()

class ConnectivityTestController(Controller):
    """
    Controller for the connectivity test window
    
    I perform some basic tests in order to find out where the network
    problem is:
     1. Check if ppp0 is up
     2. Check if we can ping the gateway
     3. Check if we can ping our DNS
     4. Check if our DNS resolves www.google.com
     5. Check if we can fetch google's homepage 
    """
    
    def __init__(self, model, parent_ctrl):
        Controller.__init__(self, model)
        self.parent_ctrl = parent_ctrl
        
        from cStringIO import StringIO
        self.report = StringIO()
    
    def register_view(self, view):
        Controller.register_view(self, view)
        self.do_test()
    
    def set_and_save_outcome(self, message, result, done=False):
        # paint result in view
        self.view.set_outcome(message, result)
        
        # save result in report
        outcome = result and _('OK') or _('ERROR')
        newline = '\n%s\t%s' % (message, outcome)
        self.report.write(newline)
        
        # if test was unsuccessful or we are done, enable save button
        if not result or done:
            self.view['save_button'].set_sensitive(True)
            self.report.write('\n') # be nice
    
    def do_test(self):
        # get and save the initial report
        def report_callback(report):
            self.report.write(report)
            # now do the conn test
            self._do_connectivity_test()
        
        self.model.get_support_report().addCallback(report_callback)
    
    def _do_connectivity_test(self):
        # test one, check if ppp is up
        message1 = _("Checking if ppp interface is up...")
        self.view.set_status(message1)
        test1_def = self.model.get_ppp_link_status()
        def ppp_status_cb(status):
            self.set_and_save_outcome(message1, status)
            if not status:
                return
            
            self.view['progressbar1'].set_fraction(.2)
            
            # test two, check if we can ping our gateway
            message2 = _("Checking if we can reach gateway...")
            self.view.set_status(message2)
            test2_def = self.model.check_if_we_can_reach_gateway()
            def reach_gateway_cb(response):
                self.set_and_save_outcome(message2, response)
                if not response:
                    return
                self.view['progressbar1'].set_fraction(.4)
            
                # test three, check if we have connectivity try to ping
                # the nameserver
                message3 = _("Checking if we can reach nameserver...")
                self.view.set_status(message3)
                test3_def = self.model.check_if_we_can_reach_dns()
                def reach_dns_callback(response):
                    self.set_and_save_outcome(message3, response)
                    if not response:
                        return
                    self.view['progressbar1'].set_fraction(.6)
                    
                    # test four check if DNS is working
                    message4 = _("Checking if we can resolve google.com...")
                    self.view.set_status(message4)
        
                    def resolve_callback(response):
                        self.set_and_save_outcome(message4, True)
                        self.view['progressbar1'].set_fraction(.8)
                        
                        # test five, try to fetch a page
                        message5 = _(
                             "Checking if we can fetch google's homepage...")
                        self.view.set_status(message5)
                        def fetch_cb(response):
                            self.set_and_save_outcome(message5, True,
                                                      done=True)
                            self.view['progressbar1'].set_fraction(1)
                            self.view['save_button'].set_sensitive(True)
                            self.view.set_title(_("Connectivity test done!"))
                        
                        def fetch_eb(failure):
                            self.set_and_save_outcome(message5, False)
                        
                        self.model.fetch_google_homepage().addCallbacks(
                                                        fetch_cb, fetch_eb)
                    def resolve_errback(failure):
                        self.set_and_save_outcome(message4, False)
                    
                    self.model.check_if_dns_resolves().addCallbacks(
                                resolve_callback, resolve_errback)
                test3_def.addCallback(reach_dns_callback)
            test2_def.addCallback(reach_gateway_cb)
        test1_def.addCallback(ppp_status_cb)
    
    def on_save_button_clicked(self, widget):
        import os.path
        import vmc.utils.dialogs, vmc.utils.globals
        from vmc.utils.utilities import save_file
        # we propose a path to the user, this could be a candidate for OAL
        proposed_path = os.path.join(os.path.expanduser('~'),
                        'Desktop',
                        _('%s-report.txt') % vmc.utils.globals.APP_SHORT_NAME)
        path = vmc.utils.dialogs.save_standard_file(proposed_path)
        if path:
            save_file(path, self.report.getvalue())
            self.report.close()
            self._hide_myself()
    
    def on_close_button_clicked(self, widget):
        self._hide_myself()
    
    def _hide_myself(self):
        self.model.unregister_observer(self)
        self.view.hide()
    