# -*- coding: utf-8 -*-
# Copyright (C) 2006-2007  Vodafone España, S.A.
# Author:  Pablo Martí
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
Preferences' Controllers
"""
__version__ = "$Rev: 658 $"

import os

from twisted.python import log

from vmc.gtkmvc import Controller
from vmc.gtkmvc import Model

from vmc.common.hardware import CONN_OPTS_DICT
from vmc.common.config import VMCConfig
from vmc.common.oal import get_os_object
from vmc.common import wvdialprofile
import vmc.common.exceptions as ex
from vmc.common.encoding import _

from vmc.utils.ValidatedEntry import ValidatedEntry, v_ip
import vmc.utils.globals
import vmc.utils.dialogs

from vmc.models.preferences import VALIDITY_DICT
from vmc.views.preferences import ProfileEditorView

class PreferencesController(Controller):
    """Controller for preferences"""
    
    def __init__(self, model, parent_ctrl):
        Controller.__init__(self, model)
        self.parent_ctrl = parent_ctrl
        # initial_conn is a dictionary that will cache the initial values
        # in the GUI to keep track of what have changed
        self.initial_conn = None
        # TODO: One day this could live in the glade file, we need to add
        # it as a custom widget to glade3
        self.dns1_entry = ValidatedEntry(v_ip)
        self.dns2_entry = ValidatedEntry(v_ip)
    
    def register_view(self, view):
        Controller.register_view(self, view)
        self._setup()
    
    def _setup(self):
        # setup preferences stuff
        preferences = self.get_conn_preferences()
        self.initial_conn = preferences['connection']

    def get_conn_preferences(self):
        model = self.view['connection_combobox'].get_model()
        index = self.view['connection_combobox'].get_active()
        if index < 0:
            conn = None
        else:
            conn = model[index][0]
        
        conn_option = CONN_OPTS_DICT[conn]
        return dict(username=self.view['username_entry'].get_text(),
                    password=self.view['password_entry'].get_text(),
                    connection=conn_option,
                    apn = self.view['apn_entry'].get_text())
    
    def get_selected_wvdial_profile(self):
        model = self.view['wvdial_profiles_combobox'].get_model()
        index = self.view['wvdial_profiles_combobox'].get_active()
        if index < 0:
            return None
        else:
            name = model[index][0]
            return wvdialprofile.get_profile_from_name(name)
    
    # ------------------------------------------------------------ #
    #                       Signals Handling                       #
    # ------------------------------------------------------------ #
    
    def on_preferences_ok_button_clicked(self, widget):
        preferences = self.get_conn_preferences()
        
        conf = VMCConfig()
        conf.set('connection', 'username', preferences['username'])
        conf.set('connection', 'password', preferences['password'])
        conf.set('connection', 'connection', preferences['connection'])
        conf.set('connection', 'apn', preferences['apn'])
        
        if self.view['dns_checkbutton'].get_active():
            dns1 = self.dns1_entry.get_text()
            dns2 = self.dns2_entry.get_text()
            if not self.dns1_entry.isvalid() or not self.dns2_entry.isvalid():
                return
            else:
                conf.set('connection', 'dns1', dns1)
                conf.set('connection', 'dns2', dns2)
                conf.set('connection', 'staticdns', 'yes')
                
        else:
            conf.set('connection', 'staticdns', 'no')
        
        if get_os_object().manage_secrets:
            # only if its necessary
            bool = \
               self.view['secrets_checkbutton'].get_active() and 'yes' or 'no'
            conf.set('preferences', 'manage_secrets', bool)
        
        # secondpage
        custom = self.view['custom_profile_checkbutton'].get_active()
        if custom:
            conf.set('connection', 'use_custom_wvdial_profile', 'yes')
            # get combobox option
            profile = self.get_selected_wvdial_profile()
            if not profile:
                conf.set('connection', 'use_custom_wvdial_profile', 'no')
            else:
                conf.set('connection', 'use_custom_wvdial_profile', 'yes')
                conf.set('connection', 'wvdial_profile_name', profile.name)
                
        else:
            conf.set('connection', 'use_custom_wvdial_profile', 'no')
        
        conf.write()
        
        from vmc.common.wvdialprofile import link_configuration
        link_configuration()
        
        if not self.model.device:
            self._hide_ourselves()
            return
        
        # check if the user has changed the conn preferences
        if self.initial_conn == preferences['connection']:
            self._hide_ourselves()
            return
        
        # now set the new connection preference
        conn_str = self.model.device.conn_dict[preferences['connection']]
        d = self.model.send_conn_string(conn_str)
        def callback(resp):
            # now hide
            self._hide_ourselves()
            self.parent_ctrl.set_cell_type()
            
        def errback(failure):
            log.err(failure, "error received setting up connection")
            
        d.addCallback(callback)
        d.addErrback(errback)
    
    def on_preferences_cancel_button_clicked(self, widget):
        self._hide_ourselves()
    
    def on_help_button_clicked(self, widget):
        message = _("System information")
        details = _("""
%(name)s relies on
wvdial and pppd in order to connect to Internet. The
behaviour of this packages varies between distros as
they patch the source. Toggle this checkbutton if you
want %(name)s to
manage pppd's password files. In some distros like
SLED 10, this is not configurable.
""") % dict(name=vmc.utils.globals.APP_LONG_NAME)
        vmc.utils.dialogs.open_message_dialog(message, details)
    
    def _hide_ourselves(self):
        self.model.unregister_observer(self)
        self.view.hide()
    
    def on_dns_checkbutton_toggled(self, widget):
        try:
            active = self.view['dns_checkbutton'].get_active()
        except TypeError:
            # We ignore this because the view will emit the toggled signal
            # and as the controller is not still registered, it produces
            # an exception, we can safely ignore it.
            return
        
        if active:
            self.dns1_entry.set_sensitive(True)
            self.dns2_entry.set_sensitive(True)
        else:
            self.dns1_entry.set_sensitive(False)
            self.dns2_entry.set_sensitive(False)
    
    # second notebook page stuff
    
    def on_custom_profile_checkbutton_toggled(self, button):
        try:
            if button.get_active():
                self.view['frame19'].set_sensitive(True)
                # populate combobox
                self.view.setup_wvdial_combobox()
            else:
                self.view['frame19'].set_sensitive(False)
        except TypeError:
            # the controller is not registered yet,
            # it can be safely ignored
            pass
    
    def on_new_wvdialprof_button_clicked(self, button):
        ctrl = ProfileEditorController(Model())
        view = ProfileEditorView(ctrl)
        view.set_new_view()
    
    def on_edit_wvdialprof_button_clicked(self, button):
        profile = self.get_selected_wvdial_profile()
        if not profile:
            return
        
        ctrl = ProfileEditorController(Model())
        view = ProfileEditorView(ctrl)
        view.set_edit_view()
        
        ctrl._load_profile(profile)
    
    def on_delete_wvdialprof_button_clicked(self, button):
        profile = self.get_selected_wvdial_profile()
        if not profile:
            return
        
        message = _("Delete profile")
        details = _(
            "Are you sure that you want to delete profile %s") % profile.name
        resp = vmc.utils.dialogs.open_warning_request_cancel_ok(message,
                                                                details)
        if not resp:
            return
        
        try:
            wvdialprofile.delete_profile(profile)
        except ex.ProfileNotFoundError:
            message = _("Error while deleting profile")
            details = _("Profile %s does not exists") % profile
            vmc.utils.dialogs.open_warning_dialog(message, details)
        except (IOError, OSError):
            message = _("Error while deleting profile")
            details = _("Profile %s can not be deleted") % profile
            vmc.utils.dialogs.open_warning_dialog(message, details)

        # regenerate combobox
        self.view.setup_wvdial_combobox()

class ProfileEditorController(Controller):
    """Controller for the profiles' editor"""
    
    def __init__(self, model, profile=None):
        Controller.__init__(self, model)
        self.changed = False
        self.setting_text = False
        self.profile = profile
    
    def register_view(self, view):
        Controller.register_view(self, view)
        self.setup_signals()
        if self.profile:
            self._load_profile(self.profile)
        
        self.view['save_button'].set_sensitive(False)
    
    def setup_signals(self):
        textbuffer = self.view['textview1'].get_buffer()
        textbuffer.connect('changed', self._textbuffer_changed)
    
    def _textbuffer_changed(self, textbuffer):
        if self.setting_text or self.changed:
            return
        
        self.changed = True
        self.view['save_button'].set_sensitive(True)
    
    def on_save_button_clicked(self, button):
        from vmc.utils.utilities import save_file
        text = self.get_text()
        if self.profile: # we are editing an existing profile
            save_file(self.profile.path, text)
            self._hide_myself()
        else: # we are editing a new profile
            # this path.join is needed as the FileDialog does a basename(path)
            path = vmc.utils.dialogs.save_standard_file(os.path.join(
                        vmc.utils.globals.WVDIAL_PROFILES, 'foo'))
            if path:
                save_file(path, text)
                self._hide_myself()
    
    def _hide_myself(self):
        self.view.hide()
        self.model.unregister_observer(self)
        
    def on_cancel_button_clicked(self, button):
        self.view.hide()
        self.model.unregister_observer(self)
    
    def _load_profile(self, profile):
        from vmc.utils.utilities import get_file_data
        text = get_file_data(profile.path)
        self.set_text(text)
        self.profile = profile
    
    def get_text(self):
        textbuffer = self.view['textview1'].get_buffer()
        start, end = textbuffer.get_bounds()
        return textbuffer.get_text(start, end)
    
    def set_text(self, text):
        textbuffer = self.view['textview1'].get_buffer()
        self.setting_text = True
        textbuffer.set_text(text)
        self.setting_text = False

class SMSPreferencesController(Controller):
    
    def __init__(self, model):
        Controller.__init__(self, model)
        self.initial_smsc = None
    
    def register_view(self, view):
        Controller.register_view(self, view)
        self._setup()
    
    def _setup(self):
        d = self.model.get_smsc()
        def callback(smsc):
            if not smsc:
                return
            
            # show it in the view and remember it
            self.view['smsc_entry'].set_text(smsc)
            self.initial_smsc = smsc
            
            # now try to obtain the operator this name corresponds to
            from vmc.utils.utilities import (get_network_name_from_smsc,
                                         get_all_smsc)
            
            def callback(resp):
                self.view.populate_smsc_combobox(resp)
                get_network_name_from_smsc(smsc).addCallback(
                                                 self._process_smsc)
            get_all_smsc().addCallback(callback)
        d.addCallback(callback)
        
        self._setup_message_options()
    
    def _setup_message_options(self):
        conf = VMCConfig()
        validity = conf.get('sms', 'validity')
        if not validity:
            conf.get('sms', 'validity', 'maximum')
            conf.write()
            validity = 'maximum'
        
        combobox = self.view['validity_combobox']
        model = combobox.get_model()
        count = 0
        for row in model:
            option = row[0]
            if validity == VALIDITY_DICT[option]:
                combobox.set_active(count)
                return    
            count += 1
        
    def _hide_myself(self):
        self.view.hide()
        self.model.unregister_observer(self)
    
    def _process_smsc(self, smscobj):
        combobox = self.view['combobox1']
        model = combobox.get_model()
        if not smscobj:
            # we haven't find a SMSC
            from vmc.common.persistent import SMSCListItem
            item = SMSCListItem(self.initial_smsc, _('Custom'))
            iter = model.add_smsc(item)
            combobox.set_active_iter(iter)
        else:
            count = 0
            for row in model:
                if smscobj == row[0]:
                    combobox.set_active(count)
                    return
                count += 1
    
    def on_combobox1_changed(self, combobox):
        smscobj = self._get_active_combobox_item('combobox1')
        if smscobj:
            self.view['smsc_entry'].set_text(smscobj.number)
    
    def _get_active_combobox_item(self, comboname):
        combobox = self.view[comboname]
        model = combobox.get_model()
        active = combobox.get_active()
        if active < 0:
            return None
        
        return model[active][0]
        
    def on_ok_button_clicked(self, widget):
        # save message options
        conf = VMCConfig()
        validity = self._get_active_combobox_item('validity_combobox')
        if validity:
            validity_key = VALIDITY_DICT[validity]
            conf.set('sms', 'validity', validity_key)
            conf.write()
        # check that we have changed the SMSC info
        smscnumber = self.view['smsc_entry'].get_text()
        if self.initial_smsc != smscnumber:
            d = self.model.set_smsc(smscnumber)
            d.addCallback(lambda x: self._hide_myself())
        else:
            self._hide_myself()
    
    def on_cancel_button_clicked(self, widget):
        self._hide_myself()
    