# -*- coding: utf-8 -*-
# Copyright (C) 2006-2007  Vodafone España, S.A.
# Author:  Pablo Martí
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""Controllers for the sms dialogs"""
__version__ = "$Rev: 616 $"

import os
import datetime

import gtk

from twisted.internet.defer import DeferredList, Deferred, succeed

from vmc.gtkmvc import Controller
from vmc.common.smsng.sms import (ShortMessageSubmit, check_if_ucs2,
                                  MAX_LENGTH_7BIT, MAX_LENGTH_UCS2)
from vmc.controllers.base import TV_DICT, TV_DICT_REV
from vmc.utils.ValidatedEntry import ValidatedEntry, v_phone
import vmc.utils
from vmc.common.encoding import _

MOBILE_IMG = gtk.gdk.pixbuf_new_from_file(
          os.path.join(vmc.utils.globals.IMAGES_DIR, 'mobile.png'))
COMPUTER_IMG = gtk.gdk.pixbuf_new_from_file(
              os.path.join(vmc.utils.globals.IMAGES_DIR, 'computer.png'))

class NewSmsController(Controller):
    """Controller for the new sms dialog"""
    
    def __init__(self, model, parent_ctrl = None):
        Controller.__init__(self, model)
        self.parent_ctrl = parent_ctrl
        self.max_length = MAX_LENGTH_7BIT
        self.numbers_entry = ValidatedEntry(v_phone)
    
    def register_view(self, view):
        Controller.register_view(self, view)
        # set initial text
        self.view['bottom_label'].set_text(_('0/%d chars') % self.max_length)
        # signals stuff
        textbuffer = self.view['sms_edit_text_view'].get_buffer()
        textbuffer.connect('changed', self._textbuffer_changed)
        # show up
        self.numbers_entry.grab_focus()
    
    def on_send_button_clicked(self, widget):
        # check that is a valid number
        if not self.numbers_entry.isvalid():
            self.numbers_entry.grab_focus()
            return
        
        def get_sms_list_cb(messages):
            if not messages:
                return
        
            message_text = self.get_message_text()
            if not message_text:
                resp = vmc.utils.dialogs.open_warning_request_cancel_ok(
                        _("Send empty message"),
                        _("Are you sure you want to send an empty message?"))
                if not resp: # user said no
                    return
            
            # send the sms
            sms_cb_list = []
            smslist = []
            for sms in messages:
                sms_cb_list.append(self.parent_ctrl.model.send_sms(sms))
                smslist.append(sms)
                self.view['sms_progressbar'].set_fraction(
                                      len(smslist) / len(messages))
            
            def list_cb(resp):
                self.save_messages_to_db(smslist,
                                         where=TV_DICT_REV['sent_treeview'])
                self.model.unregister_observer(self)
                self.view.hide()
            
            dlist = DeferredList(sms_cb_list)
            dlist.addCallback(list_cb)
        
        self.get_messages_list().addCallback(get_sms_list_cb)
        
    def on_save_button_clicked(self, widget):
        # get number list and text
        d = self.get_messages_list()
        def get_sms_list_cb(messages):
            if not messages:
                return
            
            self.save_messages_to_db(messages)
            
            self.model.unregister_observer(self)
            self.view.hide()
        
        d.addCallback(get_sms_list_cb)
    
    def on_cancel_button_clicked(self, widget):
        self.model.unregister_observer(self)
        self.view.hide()
    
    def on_contacts_button_clicked(self, widget):
        from vmc.controllers.contacts import ContactsListController
        from vmc.views.contacts import ContactsListView
        from vmc.gtkmvc.model import Model
        
        ctrl = ContactsListController(Model(), self)
        view = ContactsListView(ctrl)
        resp = view.run()
    
    def _textbuffer_changed(self, textbuffer):
        """Handler for the textbuffer changed signal"""
        text = self.get_message_text()
        self.max_length = check_if_ucs2(text) \
                    and MAX_LENGTH_UCS2 or MAX_LENGTH_7BIT
        
        charcount = textbuffer.get_char_count()
        if charcount > self.max_length:
            start, end = textbuffer.get_bounds()
            message_text = textbuffer.get_text(start, end)
            textbuffer.set_text(message_text[:self.max_length])
            charcount = self.max_length
        
        self.view['bottom_label'].set_text(
            _('%(chrcnt)d/%(maxchr)d chars') % dict(chrcnt=charcount,
                                                    maxchr=self.max_length))
    
    def get_message_text(self):
        """Returns the text written by the user in the textview"""
        textbuffer = self.view['sms_edit_text_view'].get_buffer()
        bounds = textbuffer.get_bounds()
        return textbuffer.get_text(bounds[0], bounds[1])
    
    def get_numbers_list(self):
        """Returns a list with all the numbers in the recipients_entry"""
        numbers_text = self.numbers_entry.get_text()
        numbers = numbers_text.split(',')
        if numbers[0] == '':
            return None
        
        return numbers
        
    def get_messages_list(self):
        """Returns a list with all the messages to send/save"""
        # get number list and text
        numbers = self.get_numbers_list()
        if not numbers:
            return succeed([])
        
        message_text = self.get_message_text()
        message_text = unicode(message_text, 'utf8')
        
        d = self.parent_ctrl.model.get_smsc()
        def callback(smsc):
            return [ShortMessageSubmit(number, message_text,
                    datetime=datetime.datetime.now(), smsc=smsc)
                    for number in numbers]
        
        d.addCallback(callback)
        d2 = Deferred()
        d.chainDeferred(d2)
        return d2
    
    def set_entry_text(self, text):
        self.numbers_entry.set_text(text)
    
    def save_messages_to_db(self, smslist,
                            where=TV_DICT_REV['drafts_treeview']):
        
        from vmc.common.messages import Messages
        messages = Messages(self.parent_ctrl.model.get_sconn())
        def callback(smslistback):
            tv_name = TV_DICT[where]
            model = self.parent_ctrl.view[tv_name].get_model()
            model.add_messages(smslistback)
            messages.close()
        
        messages.add_messages(smslist, where).addCallback(callback)

class ForwardSmsController(NewSmsController):
    """Controller for ForwardSms"""
    
    def __init__(self, model, parent_ctrl):
        NewSmsController.__init__(self, model, parent_ctrl)
    
    def set_recipient_numbers(self, text):
        self.numbers_entry.set_text(text)
    
    def set_textbuffer_text(self, text):
        textbuffer = self.view['sms_edit_text_view'].get_buffer()
        textbuffer.set_text(text)
        