# -*- coding: utf-8 -*-
# Copyright (C) 2006-2007  Vodafone España, S.A.
# Author:  Pablo Martí
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""Model for main application"""
__version__ = "$Rev: 658 $"

import os

from twisted.internet import defer, reactor
from twisted.python.failure import Failure
from twisted.python.procutils import which
from twisted.python import log

from vmc.gtkmvc import Model
from vmc.common.config import VMCConfig
from vmc.common.oal import get_os_object, mark_for_oal
import vmc.common.exceptions as ex
import vmc.utils.globals

from vmc.models.base import SerialConnectionDispatcher

class ApplicationModel(Model, SerialConnectionDispatcher):
    """Model for the main application"""
    
    __properties__ = {
          "connecting" : False,
          "internet_connected" : False,
          "usage_shown": False,
          "sms_shown" : True,
          "support_shown" : False
    }
    
    def __init__(self, sview, device=None):
        Model.__init__(self)
        SerialConnectionDispatcher.__init__(self, device)
        self.sview = sview
        self.iconn = None
        self.netspeed = None
        self.wvproto = None
        self.wvdial_conf_path = None
        self.connecting = False
        self.internet_connected = False
        
    #----------------------------------------------#
    # CONNECTION/DISCONNECTION SERIAL/INTERNET     #
    #----------------------------------------------#
    
    def connect_internet(self):
        """Starts wvdial in case it wasn't already started"""
        d = defer.Deferred()
        
        if self.connecting:
            return d.errback(Failure(ex.AlreadyConnecting))
        self.connecting = True
        
        if self.internet_connected:
            return d.errback(Failure(ex.AlreadyConnected))
        
        config = VMCConfig()
        
        if config.getboolean('preferences', 'manage_secrets'):
            self.setup_secrets()
        
        self.setup_connection()
        
        from vmc.common.config import get_wvdial_conf_file
        from vmc.common.netspeed import NetworkSpeed
        
        wvdial_path = which('wvdial')[0]
        self.wvdial_conf_path = get_wvdial_conf_file(self.device.dport)
            
        # cleanup just in case
        try:
            os.unlink(vmc.utils.globals.VMC_DNS_LOCK)
        except:
            pass
        
        staticdns = config.getboolean('connection', 'staticdns')
        if staticdns:
            from vmc.utils.utilities import generate_vmc_dns_lock
            dns1 = config.get('connection', 'dns1')
            dns2 = config.get('connection', 'dns2')
            generate_vmc_dns_lock(dns1, dns2)
        
        config.close()
        
        # some distros like SuSe eliminated the setuid bit from pppd
        # in this distros we will need to invoke wvdial with gksudo
        # get_wvdial_args abstracts it for us
        osobj = get_os_object()
        args = osobj.get_connection_args(self.wvdial_conf_path)
            
        self.netspeed = NetworkSpeed()
        def callback(ignored):
            self.internet_connected = True
            self.connecting = False
            self.netspeed.start()
            d.callback('connected')
        
        from vmc.common.protocol import WVDialProtocol
        self.wvproto = WVDialProtocol(staticdns)
        self.wvproto.deferred.addCallback(callback)
        self.iconn = reactor.spawnProcess(self.wvproto, args[0], args,
                                          env=None)
        return d
    
    def disconnect_internet(self):
        """Disconnects from the Internet and kills all related processes"""
        if self.internet_connected or self.connecting:
            try:
                self.netspeed.stop()
            except (AssertionError, AttributeError):
                pass
            
            self.netspeed = None
            
            osobj = get_os_object()
            args = osobj.get_disconnection_args()
            # ignore that pppd is gonna die
            self.wvproto.ignore_disconnect = True
            reactor.spawnProcess(self.wvproto, args[0], args, env=None)
            
            # cleanup the created wvdial.conf and its dir
            try:
                path = os.path.dirname(self.wvdial_conf_path)
                os.unlink(self.wvdial_conf_path)
                os.rmdir(path)
            except IOError:
                pass
            
            try:
                os.unlink(vmc.utils.globals.VMC_DNS_LOCK)
            except:
                pass
            
            self.wvdial_conf_path = None
            self.internet_connected = False
            self.connecting = False
            
    def setup_connection(self):
        """
        Sets up the internet connection
        """
        config = VMCConfig()
        preferred = config.get('connection', 'connection')
        config.close()
        
        conn_str = self.device.conn_dict[preferred]
        d = self.send_conn_string(conn_str)
        def callback(resp):
            pass
        def errback(failure):
            log.err(failure, "error setting up connection")
        
        d.addCallback(callback)
        d.addErrback(errback)
    
    def setup_secrets(self):
        # get username and password
        config = VMCConfig()
        username = config.get('connection', 'username')
        password = config.get('connection', 'password')
        config.close()
        #setup secrets
        from vmc.common.passwordfd import (SecretsEntry,
                                           append_username_passwd_to_secret)
        entry = SecretsEntry(username, '*', password)
        append_username_passwd_to_secret(mark_for_oal('PAP_SECRETS'), entry)
        append_username_passwd_to_secret(mark_for_oal('CHAP_SECRETS'), entry)
    
    #----------------------------------------------#
    # EXTRA FUNCTIONALITY                          #
    #----------------------------------------------#
    
    def set_splash_text(self, text):
        self.sview.set_text(text)
    
    def set_splash_fraction(self, fraction):
        self.sview.set_fraction(fraction)
    