# -*- coding: utf-8 -*-
# Copyright (C) 2006-2007  Vodafone España, S.A.
# Author:  Pablo Martí
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""Model for the diagnostics window"""
__version__ = "$Rev: 601 $"

import os

from twisted.internet import defer, reactor
from twisted.internet.utils import getProcessOutput

from vmc.gtkmvc import Model
from vmc.models.base import SerialConnectionDispatcher
from vmc.utils.ifconfig import ifconfig
import vmc.utils.globals

class DiagnosticsModel(Model, SerialConnectionDispatcher):
    """Model for diagnostics window"""
    
    __properties__ = {}
    
    def __init__(self, device):
        Model.__init__(self)
        SerialConnectionDispatcher.__init__(self, device)
    
    def get_uptime(self):
        d = getProcessOutput('cat', args=['/proc/uptime'])
        def2 = defer.Deferred()
        def callback(uptime, d2):
            uptime = uptime.split()[0]
            uptime = float(uptime)
            from vmc.common.uptime import get_uptime_string
            msg = get_uptime_string(uptime)
            d2.callback(msg)
            
        d.addCallback(callback, def2)
        return def2
    
    def get_dns_info(self):
        result = {}
        resolv = open('/etc/resolv.conf')
        for line in resolv:
            if line.startswith('nameserver'):
                address = line.split()[1]
                if 'DNS1' not in result:
                    result['DNS1'] = address
                elif 'DNS2' not in result:
                    result['DNS2'] = address
                    break
        
        resolv.close()
        return result  
        
    def get_os_name(self):
        return os.uname()[0]
    
    def get_os_version(self):
        return os.uname()[2]
    
    def get_support_report(self):
        from string import Template
        from cStringIO import StringIO
        from vmc.utils.utilities import get_file_data
        
        filedata = get_file_data(vmc.utils.globals.REPTEMPLATE)
        data = StringIO(filedata)
        template = Template(data.getvalue())
        d = self._get_template_dict()
        def template_callback(template_dict):
            data.close()
            return template.substitute(template_dict)
        
        d.addCallback(template_callback)
        d2 = defer.Deferred()
        d.chainDeferred(d2)
        return d2
        
    def _get_template_dict(self):
        result = dict(osname=self.get_os_name(),
                      osversion=self.get_os_version(),
                      appname=vmc.utils.globals.APP_LONG_NAME,
                      appversion=vmc.utils.globals.APP_VERSION,
                      DNS1='not found',
                      DNS2='not found')
        
        result_list = []
        def update_dict_with_result(value, key):
            result[key] = value
        
        d = self.get_imei_number()
        d.addCallback(update_dict_with_result, 'imei')
        result_list.append(d)
        
        d = self.get_imsi_number()
        d.addCallback(update_dict_with_result, 'imsi')
        result_list.append(d)
        
        d = self.get_card_version()
        d.addCallback(update_dict_with_result, 'firmware')
        result_list.append(d)
        
        d = self.get_card_model()
        d.addCallback(update_dict_with_result, 'device')
        result_list.append(d)
        
        d = self.get_uptime()
        d.addCallback(update_dict_with_result, 'uptime')
        result_list.append(d)
        
        dns_dict = self.get_dns_info()
        result.update(dns_dict)
        
        # when all the results are gathered fire response
        deflist = defer.DeferredList(result_list)
        d2 = defer.Deferred()
        def list_callback(ignored):
            d2.callback(result)
        
        deflist.addCallback(list_callback)
        return d2
    
    def _get_ppp_info(self, iface='ppp0'):
        from twisted.internet import threads
        return threads.deferToThread(ifconfig, iface)
    
    def get_ppp_link_status(self):
        d = self._get_ppp_info()
        def callback(pppinfo):
            return pppinfo.get('addr', False)
        
        d.addCallback(callback)
        d2 = defer.Deferred()
        d.chainDeferred(d2)
        return d2
    
    def obtain_gateway_address(self):
        d = self._get_ppp_info()
        def callback(pppinfo):
            return pppinfo.get('raddr', False)
        
        d.addCallback(callback)
        d2 = defer.Deferred()
        d.chainDeferred(d2)
        return d2
    
    def check_if_we_can_reach_gateway(self):
        d = self.obtain_gateway_address()
        d2 = defer.Deferred()
        def callback(addr):
            def ping_callback(response):
                d2.callback(self.process_ping_output(response))
            
            getProcessOutput('ping',
                         args=['-c', '3', addr]).addCallback(ping_callback)
        
        d.addCallback(callback)
        return d2
    
    def process_ping_output(self, output):
        import re
        pattern = '(\d+\s+bytes|From\s+\d+.)'
        result = []
        for match in re.finditer(pattern, output, re.X):
            result.append(match.groups())
        return len(result)
    
    def check_if_we_can_reach_dns(self):
        dns_dict = self.get_dns_info()
        d = getProcessOutput('ping', args=['-c', '3', dns_dict['DNS1']])
        def ping_callback(response):
            return self.process_ping_output(response)
        
        d.addCallback(ping_callback)
        d2 = defer.Deferred()
        d.chainDeferred(d2)
        return d2
    
    def check_if_dns_resolves(self, stablename='www.google.com'):
        return reactor.resolve(stablename)
    
    def fetch_google_homepage(self):
        from twisted.web.client import getPage
        return getPage('http://www.google.com')
    