# -*- coding: utf-8 -*-
# Copyright (C) 2006-2007  Vodafone España, S.A.
# Author:  Pablo Martí
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA..
"""Misc utilities"""

__version__ = '$Rev: 616 $'

import re
import os
import shutil

from twisted.internet import defer
from twisted.python import log

import vmc.utils.globals
from vmc.common.persistent import NetworksManager, SMSCManager
from vmc.common.csvutils import CSVNetworkReader, CSVSMSCListReader
from vmc.common.oal import mark_for_oal

def dict_reverter(dict_in):
    """Returns a reverted copy of C{dict_in}"""
    dict_out = {}
    for k, v in dict_in.iteritems():
        dict_out[v] = k
    
    return dict_out

def get_file_data(path):
    """Returns the data of the file at path C{path}"""
    fileobj = open(path)
    try: 
        data = fileobj.read() 
    finally:
        fileobj.close()
    return data

def save_file(path, data):
    """saves data in path"""
    try:
        try:
            fileobj = open(path, 'w')
            fileobj.write(data)
        except Exception:
            raise
    finally:
        try:
            fileobj.close()
        except:
            pass

def generate_vmc_dns_lock(dns1, dns2):
    text = "DNS1  " + dns1 + '\n'
    text += "DNS2  " + dns2 + '\n'
    save_file(vmc.utils.globals.VMC_DNS_LOCK, text)

def populate_networks_db():
    """Populates the networks database"""
    
    resp_deferred = defer.Deferred()
    fobj = open(vmc.utils.globals.NETWORKS_CSV)
    iterator = CSVNetworkReader(fobj)
    
    nmana = NetworksManager()
    d = nmana.populate_networks_db(iterator.get_rows())
    
    def populate_nets_callback(ignored):
        nmana.close()
        fobj.close()
        resp_deferred.callback(True)
    
    def populate_nets_errback(failure):
        nmana.close()
        fobj.close()
        log.err(failure)
        resp_deferred.errback(failure)
    
    d.addCallbacks(populate_nets_callback, populate_nets_errback)
    
    fobj2 = open(vmc.utils.globals.SMSCLIST_CSV)
    iterator2 = CSVSMSCListReader(fobj2)
    
    smana = SMSCManager()
    d2 = smana.populate_smsc_db(iterator2.get_rows())
    def populate_smsc_callback(ignored):
        smana.close()
        fobj2.close()
        
    def populate_smsc_errback(failure):
        log.err(failure)
        smana.close()
        fobj2.close()
    
    d2.addCallbacks(populate_smsc_callback, populate_smsc_errback)
    
    return resp_deferred
    
def initial_user_setup():
    """I perform the operations needed for the initial user setup"""
    try:
        os.mkdir(vmc.utils.globals.VMC_HOME)
        os.mkdir(vmc.utils.globals.WVDIAL_PROFILES)
    except OSError:
        pass
        
    # copy initial wvdial profiles
    shutil.copy(vmc.utils.globals.CFGTEMPLATE, vmc.utils.globals.VMC_CFG)
    shutil.copy(os.path.join(vmc.utils.globals.EXTRA_DIR,
                             mark_for_oal('WVDIAL_DEFAULT_PROFILE')),
                vmc.utils.globals.DEFAULT_PROFILE)
    shutil.copy(os.path.join(vmc.utils.globals.EXTRA_DIR,
                             mark_for_oal('WVDIAL_PAP_PROFILE')),
                vmc.utils.globals.PAP_PROFILE)
    shutil.copy(os.path.join(vmc.utils.globals.EXTRA_DIR,
                             mark_for_oal('WVDIAL_CHAP_PROFILE')),
                vmc.utils.globals.CHAP_PROFILE)
    
    os.chmod(vmc.utils.globals.VMC_CFG, 0600)
    
    return populate_networks_db()

def get_network_name_from_id(netid):
    """Returns a deferred that will be fired with the response"""
    d2 = defer.Deferred()
    net_manager = NetworksManager()
    def callback(resp):
        d2.callback(resp)
        net_manager.close()
    
    net_manager.lookup_network(netid).addCallback(callback)
    return d2

def get_network_name_from_smsc(smsc):
    net_manager = SMSCManager()
    d = net_manager.lookup_smsc(smsc)
    def callback(resp):
        net_manager.close()
        return resp
    
    d.addCallback(callback)
    d2 = defer.Deferred()
    d.chainDeferred(d2)
    return d2

def get_all_smsc():
    net_manager = SMSCManager()
    d = net_manager.get_all_smsc()
    def callback(resp):
        net_manager.close()
        return resp
    
    d.addCallback(callback)
    d2 = defer.Deferred()
    d.chainDeferred(d2)
    return d2

code_to_country = {
    'af': 'Afghanistan',
    'al': 'Albania',
    'dz': 'Algeria',
    'as': 'American Samoa',
    'ad': 'Andorra',
    'ao': 'Angola',
    'ai': 'Anguilla',
    'aq': 'Antarctica',
    'ag': 'Antigua and Barbuda',
    'ar': 'Argentina',
    'am': 'Armenia',
    'aw': 'Aruba',
    'au': 'Australia',
    'at': 'Austria',
    'az': 'Azerbaijan',
    'bs': 'Bahamas',
    'bh': 'Bahrain',
    'bd': 'Bangladesh',
    'bb': 'Barbados',
    'by': 'Belarus',
    'be': 'Belgium',
    'bz': 'Belize',
    'bj': 'Benin',
    'bm': 'Bermuda',
    'bt': 'Bhutan',
    'bo': 'Bolivia',
    'ba': 'Bosnia and Herzegowina',
    'bw': 'Botswana',
    'bv': 'Bouvet Island',
    'br': 'Brazil',
    'io': 'British Indian Ocean Territory',
    'bn': 'Brunei Darussalam',
    'bg': 'Bulgaria',
    'bf': 'Burkina Faso',
    'bi': 'Burundi',
    'kh': 'Cambodia',
    'cm': 'Cameroon',
    'ca': 'Canada',
    'cv': 'Cape Verde',
    'ky': 'Cayman Islands',
    'cf': 'Central African Republic',
    'td': 'Chad',
    'cl': 'Chile',
    'cn': 'China',
    'cx': 'Christmas Island',
    'cc': 'Cocos (Keeling) Islands',
    'co': 'Colombia',
    'km': 'Comoros',
    'cg': 'Congo',
    'cd': 'Congo, The Democratic Republic of the',
    'ck': 'Cook Islands',
    'cr': 'Costa Rica',
    'ci': 'Cote D\'Ivoire',
    'hr': 'Croatia',
    'cu': 'Cuba',
    'cy': 'Cyprus',
    'cz': 'Czech Republic',
    'dk': 'Denmark',
    'dj': 'Djibouti',
    'dm': 'Dominica',
    'do': 'Dominican Republic',
    'tp': 'East Timor',
    'ec': 'Ecuador',
    'eg': 'Egypt',
    'sv': 'El Salvador',
    'gq': 'Equatorial Guinea',
    'er': 'Eritrea',
    'ee': 'Estonia',
    'et': 'Ethiopia',
    'fk': 'Falkland Islands (Malvinas)',
    'fo': 'Faroe Islands',
    'fj': 'Fiji',
    'fi': 'Finland',
    'fr': 'France',
    'gf': 'French Guiana',
    'pf': 'French Polynesia',
    'tf': 'French Southern Territories',
    'ga': 'Gabon',
    'gm': 'Gambia',
    'ge': 'Georgia',
    'de': 'Germany',
    'gh': 'Ghana',
    'gi': 'Gibraltar',
    'gr': 'Greece',
    'gl': 'Greenland',
    'gd': 'Grenada',
    'gp': 'Guadeloupe',
    'gu': 'Guam',
    'gt': 'Guatemala',
    'gn': 'Guinea',
    'gw': 'Guinea-Bissau',
    'gy': 'Guyana',
    'ht': 'Haiti',
    'hm': 'Heard Island and Mcdonald Islands',
    'va': 'Holy See (Vatican City State)',
    'hn': 'Honduras',
    'hk': 'Hong Kong',
    'hu': 'Hungary',
    'is': 'Iceland',
    'in': 'India',
    'id': 'Indonesia',
    'ir': 'Iran, Islamic Republic of',
    'iq': 'Iraq',
    'ie': 'Ireland',
    'il': 'Israel',
    'it': 'Italy',
    'jm': 'Jamaica',
    'jp': 'Japan',
    'jo': 'Jordan',
    'kz': 'Kazakstan',
    'ke': 'Kenya',
    'ki': 'Kiribati',
    'kp': 'Korea, Democratic People\'s Republic of',
    'kr': 'Korea, Republic of',
    'kw': 'Kuwait',
    'kg': 'Kyrgyzstan',
    'la': 'Lao People\'s Democratic Republic',
    'lv': 'Latvia',
    'lb': 'Lebanon',
    'ls': 'Lesotho',
    'lr': 'Liberia',
    'ly': 'Libyan Arab Jamahiriya',
    'li': 'Liechtenstein',
    'lt': 'Lithuania',
    'lu': 'Luxembourg',
    'mo': 'Macau',
    'mk': 'Macedonia, The Former Yugoslav Republic of',
    'mg': 'Madagascar',
    'mw': 'Malawi',
    'my': 'Malaysia',
    'mv': 'Maldives',
    'ml': 'Mali',
    'mt': 'Malta',
    'mh': 'Marshall Islands',
    'mq': 'Martinique',
    'mr': 'Mauritania',
    'mu': 'Mauritius',
    'yt': 'Mayotte',
    'mx': 'Mexico',
    'fm': 'Micronesia, Federated States of',
    'md': 'Moldova, Republic of',
    'mc': 'Monaco',
    'mn': 'Mongolia',
    'ms': 'Montserrat',
    'ma': 'Morocco',
    'mz': 'Mozambique',
    'mm': 'Myanmar',
    'na': 'Namibia',
    'nr': 'Nauru',
    'np': 'Nepal',
    'nl': 'Netherlands',
    'an': 'Netherlands Antilles',
    'nc': 'New Caledonia',
    'nz': 'New Zealand',
    'ni': 'Nicaragua',
    'ne': 'Niger',
    'ng': 'Nigeria',
    'nu': 'Niue',
    'nf': 'Norfolk Island',
    'mp': 'Northern Mariana Islands',
    'no': 'Norway',
    'om': 'Oman',
    'pk': 'Pakistan',
    'pw': 'Palau',
    'ps': 'Palestinian Territory, Occupied',
    'pa': 'Panama',
    'pg': 'Papua New Guinea',
    'py': 'Paraguay',
    'pe': 'Peru',
    'ph': 'Philippines',
    'pn': 'Pitcairn',
    'pl': 'Poland',
    'pt': 'Portugal',
    'pr': 'Puerto Rico',
    'qa': 'Qatar',
    're': 'Reunion',
    'ro': 'Romania',
    'ru': 'Russian Federation',
    'rw': 'Rwanda',
    'sh': 'Saint Helena',
    'kn': 'Saint Kitts and Nevis',
    'lc': 'Saint Lucia',
    'pm': 'Saint Pierre and Miquelon',
    'vc': 'Saint Vincent and the Grenadines',
    'ws': 'Samoa',
    'sm': 'San Marino',
    'st': 'Sao Tome and Principe',
    'sa': 'Saudi Arabia',
    'sn': 'Senegal',
    'sc': 'Seychelles',
    'sl': 'Sierra Leone',
    'sg': 'Singapore',
    'sk': 'Slovakia',
    'si': 'Slovenia',
    'sb': 'Solomon Islands',
    'so': 'Somalia',
    'za': 'South Africa',
    'gs': 'South Georgia and the South Sandwich Islands',
    'es': 'Spain',
    'lk': 'Sri Lanka',
    'sd': 'Sudan',
    'sr': 'Suriname',
    'sj': 'Svalbard and Jan Mayen',
    'sz': 'Swaziland',
    'se': 'Sweden',
    'ch': 'Switzerland',
    'sy': 'Syrian Arab Republic',
    'tw': 'Taiwan, Province of China',
    'tj': 'Tajikistan',
    'tz': 'Tanzania, United Republic of',
    'th': 'Thailand',
    'tg': 'Togo',
    'tk': 'Tokelau',
    'to': 'Tonga',
    'tt': 'Trinidad and Tobago',
    'tn': 'Tunisia',
    'tr': 'Turkey',
    'tm': 'Turkmenistan',
    'tc': 'Turks and Caicos Islands',
    'tv': 'Tuvalu',
    'ug': 'Uganda',
    'ua': 'Ukraine',
    'ae': 'United Arab Emirates',
    'gb': 'United Kingdom',
    'us': 'United States',
    'um': 'United States Minor Outlying Islands',
    'uy': 'Uruguay',
    'uz': 'Uzbekistan',
    'vu': 'Vanuatu',
    've': 'Venezuela',
    'vn': 'Viet Nam',
    'vg': 'Virgin Islands, British',
    'vi': 'Virgin Islands, U.S.',
    'wf': 'Wallis and Futuna',
    'eh': 'Western Sahara',
    'ye': 'Yemen',
    'yu': 'Yugoslavia',
    'zm': 'Zambia',
    'zw': 'Zimbabwe',
}
country_to_code = dict_reverter(code_to_country)

def ip_is_private(ip):
    """Returns True if the ip is a private address according to RFC1918"""
    # According to RFC1918
    # 10.0.0.0    - 10.255.255.255
    # 172.16.0.0  - 172.31.255.255
    # 192.168.0.0 - 192.168.255.255
    
    fields = tuple([int(field) for field in ip.split('.')])
    assert len(fields) == 4
    
    if (fields >= (10, 0, 0, 0) and fields <= (10, 255, 255, 255)) or \
        (fields >= (172, 16, 0, 0) and fields <= (172, 31, 255, 255)) or \
        (fields >= (192, 168, 0, 0) and fields <= (192, 168, 255, 255)):
        return True
    else:
        return False

number_pattern = re.compile('^\+\d+$')

def is_valid_number(number):
    return number_pattern.match(number) and True or False
